# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/proxyman/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "AllocationStrategyType",
    "AllocationStrategy",
    "AllocationStrategyAllocationStrategyConcurrency",
    "AllocationStrategyAllocationStrategyRefresh",
    "InboundConfig",
    "InboundHandlerConfig",
    "MultiplexingConfig",
    "OutboundConfig",
    "ReceiverConfig",
    "SenderConfig",
    "SniffingConfig",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class AllocationStrategyType(betterproto2.Enum):
    Always = 0
    """
    Always allocate all connection handlers.
    """

    Random = 1
    """
    Randomly allocate specific range of handlers.
    """

    External = 2
    """
    External. Not supported yet.
    """


@dataclass(eq=False, repr=False)
class AllocationStrategy(betterproto2.Message):
    type: "AllocationStrategyType" = betterproto2.field(
        1, betterproto2.TYPE_ENUM, default_factory=lambda: AllocationStrategyType(0)
    )

    concurrency: "AllocationStrategyAllocationStrategyConcurrency | None" = (
        betterproto2.field(2, betterproto2.TYPE_MESSAGE, optional=True)
    )
    """
    Number of handlers (ports) running in parallel.
    Default value is 3 if unset.
    """

    refresh: "AllocationStrategyAllocationStrategyRefresh | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Number of minutes before a handler is regenerated.
    Default value is 5 if unset.
    """


default_message_pool.register_message(
    "xray.app.proxyman", "AllocationStrategy", AllocationStrategy
)


@dataclass(eq=False, repr=False)
class AllocationStrategyAllocationStrategyConcurrency(betterproto2.Message):
    value: "int" = betterproto2.field(1, betterproto2.TYPE_UINT32)


default_message_pool.register_message(
    "xray.app.proxyman",
    "AllocationStrategy.AllocationStrategyConcurrency",
    AllocationStrategyAllocationStrategyConcurrency,
)


@dataclass(eq=False, repr=False)
class AllocationStrategyAllocationStrategyRefresh(betterproto2.Message):
    value: "int" = betterproto2.field(1, betterproto2.TYPE_UINT32)


default_message_pool.register_message(
    "xray.app.proxyman",
    "AllocationStrategy.AllocationStrategyRefresh",
    AllocationStrategyAllocationStrategyRefresh,
)


@dataclass(eq=False, repr=False)
class InboundConfig(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.proxyman", "InboundConfig", InboundConfig
)


@dataclass(eq=False, repr=False)
class InboundHandlerConfig(betterproto2.Message):
    tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    receiver_settings: "__common__serial__.TypedMessage | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )

    proxy_settings: "__common__serial__.TypedMessage | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.app.proxyman", "InboundHandlerConfig", InboundHandlerConfig
)


@dataclass(eq=False, repr=False)
class MultiplexingConfig(betterproto2.Message):
    enabled: "bool" = betterproto2.field(1, betterproto2.TYPE_BOOL)
    """
    Whether or not Mux is enabled.
    """

    concurrency: "int" = betterproto2.field(2, betterproto2.TYPE_INT32)
    """
    Max number of concurrent connections that one Mux connection can handle.
    """

    xudp_concurrency: "int" = betterproto2.field(3, betterproto2.TYPE_INT32)
    """
    Transport XUDP in another Mux.
    """

    xudp_proxy_udp_443: "str" = betterproto2.field(4, betterproto2.TYPE_STRING)
    """
    "reject" (default), "allow" or "skip".
    """


default_message_pool.register_message(
    "xray.app.proxyman", "MultiplexingConfig", MultiplexingConfig
)


@dataclass(eq=False, repr=False)
class OutboundConfig(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.proxyman", "OutboundConfig", OutboundConfig
)


@dataclass(eq=False, repr=False)
class ReceiverConfig(betterproto2.Message):
    port_list: "__common__net__.PortList | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    PortList specifies the ports which the Receiver should listen on.
    """

    listen: "__common__net__.IpOrDomain | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Listen specifies the IP address that the Receiver should listen on.
    """

    allocation_strategy: "AllocationStrategy | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )

    stream_settings: "__transport__internet__.StreamConfig | None" = betterproto2.field(
        4, betterproto2.TYPE_MESSAGE, optional=True
    )

    receive_original_destination: "bool" = betterproto2.field(5, betterproto2.TYPE_BOOL)

    sniffing_settings: "SniffingConfig | None" = betterproto2.field(
        7, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.app.proxyman", "ReceiverConfig", ReceiverConfig
)


@dataclass(eq=False, repr=False)
class SenderConfig(betterproto2.Message):
    via: "__common__net__.IpOrDomain | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Send traffic through the given IP. Only IP is allowed.
    """

    stream_settings: "__transport__internet__.StreamConfig | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )

    proxy_settings: "__transport__internet__.ProxyConfig | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )

    multiplex_settings: "MultiplexingConfig | None" = betterproto2.field(
        4, betterproto2.TYPE_MESSAGE, optional=True
    )

    via_cidr: "str" = betterproto2.field(5, betterproto2.TYPE_STRING)


default_message_pool.register_message("xray.app.proxyman", "SenderConfig", SenderConfig)


@dataclass(eq=False, repr=False)
class SniffingConfig(betterproto2.Message):
    enabled: "bool" = betterproto2.field(1, betterproto2.TYPE_BOOL)
    """
    Whether or not to enable content sniffing on an inbound connection.
    """

    destination_override: "list[str]" = betterproto2.field(
        2, betterproto2.TYPE_STRING, repeated=True
    )
    """
    Override target destination if sniff'ed protocol is in the given list.
    Supported values are "http", "tls", "fakedns".
    """

    domains_excluded: "list[str]" = betterproto2.field(
        3, betterproto2.TYPE_STRING, repeated=True
    )

    metadata_only: "bool" = betterproto2.field(4, betterproto2.TYPE_BOOL)
    """
    Whether should only try to sniff metadata without waiting for client input.
    Can be used to support SMTP like protocol where server send the first
    message.
    """

    route_only: "bool" = betterproto2.field(5, betterproto2.TYPE_BOOL)


default_message_pool.register_message(
    "xray.app.proxyman", "SniffingConfig", SniffingConfig
)


from ...common import net as __common__net__
from ...common import serial as __common__serial__
from ...transport import internet as __transport__internet__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proxy/freedom/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "ConfigDomainStrategy",
    "Config",
    "DestinationOverride",
    "Fragment",
    "Noise",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class ConfigDomainStrategy(betterproto2.Enum):
    AS_IS = 0

    USE_IP = 1

    USE_IP4 = 2

    USE_IP6 = 3

    USE_IP46 = 4

    USE_IP64 = 5

    FORCE_IP = 6

    FORCE_IP4 = 7

    FORCE_IP6 = 8

    FORCE_IP46 = 9

    FORCE_IP64 = 10


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    domain_strategy: "ConfigDomainStrategy" = betterproto2.field(
        1, betterproto2.TYPE_ENUM, default_factory=lambda: ConfigDomainStrategy(0)
    )

    destination_override: "DestinationOverride | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )

    user_level: "int" = betterproto2.field(4, betterproto2.TYPE_UINT32)

    fragment: "Fragment | None" = betterproto2.field(
        5, betterproto2.TYPE_MESSAGE, optional=True
    )

    proxy_protocol: "int" = betterproto2.field(6, betterproto2.TYPE_UINT32)

    noises: "list[Noise]" = betterproto2.field(
        7, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message("xray.proxy.freedom", "Config", Config)


@dataclass(eq=False, repr=False)
class DestinationOverride(betterproto2.Message):
    server: "__common__protocol__.ServerEndpoint | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.proxy.freedom", "DestinationOverride", DestinationOverride
)


@dataclass(eq=False, repr=False)
class Fragment(betterproto2.Message):
    packets_from: "int" = betterproto2.field(1, betterproto2.TYPE_UINT64)

    packets_to: "int" = betterproto2.field(2, betterproto2.TYPE_UINT64)

    length_min: "int" = betterproto2.field(3, betterproto2.TYPE_UINT64)

    length_max: "int" = betterproto2.field(4, betterproto2.TYPE_UINT64)

    interval_min: "int" = betterproto2.field(5, betterproto2.TYPE_UINT64)

    interval_max: "int" = betterproto2.field(6, betterproto2.TYPE_UINT64)


default_message_pool.register_message("xray.proxy.freedom", "Fragment", Fragment)


@dataclass(eq=False, repr=False)
class Noise(betterproto2.Message):
    length_min: "int" = betterproto2.field(1, betterproto2.TYPE_UINT64)

    length_max: "int" = betterproto2.field(2, betterproto2.TYPE_UINT64)

    delay_min: "int" = betterproto2.field(3, betterproto2.TYPE_UINT64)

    delay_max: "int" = betterproto2.field(4, betterproto2.TYPE_UINT64)

    packet: "bytes" = betterproto2.field(5, betterproto2.TYPE_BYTES)


default_message_pool.register_message("xray.proxy.freedom", "Noise", Noise)


from ...common import protocol as __common__protocol__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: transport/internet/headers/dns/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = ("Config",)

from dataclasses import dataclass

import betterproto2

from ......message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    domain: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.transport.internet.headers.dns", "Config", Config
)



from pydantic import Field

from xray_cfo.models.xray.api import XApi
from xray_cfo.models.xray.dns import XDns
from xray_cfo.models.xray.fake_dns import XFakeDns
from xray_cfo.models.xray.inbound import XInboundObject
from xray_cfo.models.xray.log import XLog
from xray_cfo.models.xray.metrics import XMetrics
from xray_cfo.models.xray.observatory import XBurstObservatory, XObservatory
from xray_cfo.models.xray.outbound import XOutboundObject
from xray_cfo.models.xray.policy import XPolicy
from xray_cfo.models.xray.reverse import XReverse
from xray_cfo.models.xray.routing import XRouting
from xray_cfo.models.xray.transport import XStreamSettings

from ._base import XBase


class XrayConfig(XBase):
    log: XLog | None = None
    api: XApi | None = None
    dns: XDns | None = None
    routing: XRouting | None = None
    policy: XPolicy | None = None
    inbounds: list[XInboundObject]
    outbounds: list[XOutboundObject]
    transport: XStreamSettings | None = None
    stats: dict | None = Field(default_factory=dict)
    reverse: XReverse | None = None
    fakedns: XFakeDns | list[XFakeDns] | None = None
    metrics: XMetrics | None = None
    observatory: XObservatory | None = None
    burst_observatory: XBurstObservatory | None = Field(
        None, serialization_alias="burstObservatory"
    )



from enum import StrEnum

from pydantic import Field

from xray_cfo.models.xray.transport import XStreamSettings

from .._base import XBase
from .dokodemo import XDokoDemoIn
from .http import XHttpIn
from .shadowsocks import XShadowsocksIn
from .socks import XSocksIn
from .trojan import XTrojanIn
from .vless import XVlessIn
from .vmess import XVmessIn
from .wireguard import WireGuardIn


class InProtocols(StrEnum):
    dokodemo = "dokodemo-door"
    http = "http"
    shadowsocks = "shadowsocks"
    mixed = "mixed"
    vless = "vless"
    vmess = "vmess"
    trojan = "trojan"
    wireguard = "wireguard"


class SniffingDest(StrEnum):
    http = "http"
    tls = "tls"
    quic = "quic"
    fakedns = "fakedns"
    others = "fakedns+others"


def get_default_sniffing_dest():
    return [SniffingDest.http, SniffingDest.tls]


class SniffingObject(XBase):
    enabled: bool = True
    dest_override: list[SniffingDest] | None = Field(
        default_factory=get_default_sniffing_dest, serialization_alias="destOverride"
    )
    metadata_only: bool = Field(False, serialization_alias="metadataOnly")
    domains_excluded: list[str] | None = Field(
        None, serialization_alias="domainsExcluded"
    )
    route_only: bool = Field(False, serialization_alias="routeOnly")


class AllocateStrategy(StrEnum):
    always = "always"
    random = "random"


class AllocateObject(XBase):
    strategy: AllocateStrategy = AllocateStrategy.always
    refresh: int = 5
    concurrency: int = 3


class XInboundObject(XBase):
    listen: str
    port: int | str
    protocol: InProtocols
    settings: (
        XDokoDemoIn
        | XHttpIn
        | XShadowsocksIn
        | XSocksIn
        | XTrojanIn
        | XVlessIn
        | XVmessIn
        | WireGuardIn
    )
    stream_settings: XStreamSettings | None = Field(
        None, serialization_alias="streamSettings"
    )
    tag: str
    sniffing: SniffingObject | None = None
    allocate: AllocateObject | None = None



from pydantic import Field

from .._base import XBase


class XGRPCSettings(XBase):
    authority: str | None = None
    service_name: str = Field("/", serialization_alias="serviceName")
    multi_mode: bool = Field(False, serialization_alias="multiMode")
    user_agent: str | None = None
    idle_timeout: int | None = None
    health_check_timeout: int | None = None
    permit_without_stream: bool | None = None
    initial_windows_size:  int | None = None



from grpclib.exceptions import GRPCError

from xray_cfo.xray.app.log.command import RestartLoggerRequest

from .._base import BaseService


class LoggerService(BaseService):

    async def restart_logger(self):
        try:
            await self.logger_stub.restart_logger(RestartLoggerRequest())
        except GRPCError:
            return



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: transport/internet/tls/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "CertificateUsage",
    "Certificate",
    "Config",
)

from dataclasses import dataclass

import betterproto2

from .....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class CertificateUsage(betterproto2.Enum):
    ENCIPHERMENT = 0

    AUTHORITY_VERIFY = 1

    AUTHORITY_ISSUE = 2


@dataclass(eq=False, repr=False)
class Certificate(betterproto2.Message):
    certificate: "bytes" = betterproto2.field(1, betterproto2.TYPE_BYTES)
    """
    TLS certificate in x509 format.
    """

    key: "bytes" = betterproto2.field(2, betterproto2.TYPE_BYTES)
    """
    TLS key in x509 format.
    """

    usage: "CertificateUsage" = betterproto2.field(
        3, betterproto2.TYPE_ENUM, default_factory=lambda: CertificateUsage(0)
    )

    ocsp_stapling: "int" = betterproto2.field(4, betterproto2.TYPE_UINT64)

    certificate_path: "str" = betterproto2.field(5, betterproto2.TYPE_STRING)
    """
    TLS certificate path
    """

    key_path: "str" = betterproto2.field(6, betterproto2.TYPE_STRING)
    """
    TLS Key path
    """

    One_time_loading: "bool" = betterproto2.field(7, betterproto2.TYPE_BOOL)
    """
    If true, one-Time Loading
    """

    build_chain: "bool" = betterproto2.field(8, betterproto2.TYPE_BOOL)


default_message_pool.register_message(
    "xray.transport.internet.tls", "Certificate", Certificate
)


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    allow_insecure: "bool" = betterproto2.field(1, betterproto2.TYPE_BOOL)
    """
    Whether or not to allow self-signed certificates.
    """

    certificate: "list[Certificate]" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, repeated=True
    )
    """
    List of certificates to be served on server.
    """

    server_name: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)
    """
    Override server name.
    """

    next_protocol: "list[str]" = betterproto2.field(
        4, betterproto2.TYPE_STRING, repeated=True
    )
    """
    Lists of string as ALPN values.
    """

    enable_session_resumption: "bool" = betterproto2.field(5, betterproto2.TYPE_BOOL)
    """
    Whether or not to enable session (ticket) resumption.
    """

    disable_system_root: "bool" = betterproto2.field(6, betterproto2.TYPE_BOOL)
    """
    If true, root certificates on the system will not be loaded for
    verification.
    """

    min_version: "str" = betterproto2.field(7, betterproto2.TYPE_STRING)
    """
    The minimum TLS version.
    """

    max_version: "str" = betterproto2.field(8, betterproto2.TYPE_STRING)
    """
    The maximum TLS version.
    """

    cipher_suites: "str" = betterproto2.field(9, betterproto2.TYPE_STRING)
    """
    Specify cipher suites, except for TLS 1.3.
    """

    fingerprint: "str" = betterproto2.field(11, betterproto2.TYPE_STRING)
    """
    TLS Client Hello fingerprint (uTLS).
    """

    reject_unknown_sni: "bool" = betterproto2.field(12, betterproto2.TYPE_BOOL)

    pinned_peer_certificate_chain_sha256: "list[bytes]" = betterproto2.field(
        13, betterproto2.TYPE_BYTES, repeated=True
    )
    """
    @Document Some certificate chain sha256 hashes.
    @Document After normal validation or allow_insecure, if the server's cert chain hash does not match any of these values, the connection will be aborted.
    @Critical
    """

    pinned_peer_certificate_public_key_sha256: "list[bytes]" = betterproto2.field(
        14, betterproto2.TYPE_BYTES, repeated=True
    )
    """
    @Document Some certificate public key sha256 hashes.
    @Document After normal validation (required), if one of certs in verified chain matches one of these values, the connection will be eventually accepted.
    @Critical
    """

    master_key_log: "str" = betterproto2.field(15, betterproto2.TYPE_STRING)

    curve_preferences: "list[str]" = betterproto2.field(
        16, betterproto2.TYPE_STRING, repeated=True
    )
    """
    Lists of string as CurvePreferences values.
    """

    verify_peer_cert_in_names: "list[str]" = betterproto2.field(
        17, betterproto2.TYPE_STRING, repeated=True
    )
    """
    @Document Replaces server_name to verify the peer cert.
    @Document After allow_insecure (automatically), if the server's cert can't be verified by any of these names, pinned_peer_certificate_chain_sha256 will be tried.
    @Critical
    """


default_message_pool.register_message("xray.transport.internet.tls", "Config", Config)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: common/protocol/headers.proto, common/protocol/server_spec.proto, common/protocol/user.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "SecurityType",
    "SecurityConfig",
    "ServerEndpoint",
    "User",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class SecurityType(betterproto2.Enum):
    UNKNOWN = 0

    AUTO = 2

    AES128_GCM = 3

    CHACHA20_POLY1305 = 4

    NONE = 5
    """
    [DEPRECATED 2023-06] 
    """

    ZERO = 6


@dataclass(eq=False, repr=False)
class SecurityConfig(betterproto2.Message):
    type: "SecurityType" = betterproto2.field(
        1, betterproto2.TYPE_ENUM, default_factory=lambda: SecurityType(0)
    )


default_message_pool.register_message(
    "xray.common.protocol", "SecurityConfig", SecurityConfig
)


@dataclass(eq=False, repr=False)
class ServerEndpoint(betterproto2.Message):
    address: "_net__.IpOrDomain | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )

    port: "int" = betterproto2.field(2, betterproto2.TYPE_UINT32)

    user: "list[User]" = betterproto2.field(3, betterproto2.TYPE_MESSAGE, repeated=True)


default_message_pool.register_message(
    "xray.common.protocol", "ServerEndpoint", ServerEndpoint
)


@dataclass(eq=False, repr=False)
class User(betterproto2.Message):
    """
    User is a generic user for all protocols.
    """

    level: "int" = betterproto2.field(1, betterproto2.TYPE_UINT32)

    email: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)

    account: "_serial__.TypedMessage | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Protocol specific account information. Must be the account proto in one of
    the proxies.
    """


default_message_pool.register_message("xray.common.protocol", "User", User)


from .. import net as _net__
from .. import serial as _serial__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proxy/vless/inbound/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "Fallback",
)

from dataclasses import dataclass

import betterproto2

from .....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    clients: "list[___common__protocol__.User]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )

    decryption: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)
    """
    Decryption settings. Only applies to server side, and only accepts "none"
    for now.
    """

    fallbacks: "list[Fallback]" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message("xray.proxy.vless.inbound", "Config", Config)


@dataclass(eq=False, repr=False)
class Fallback(betterproto2.Message):
    name: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    alpn: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)

    path: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)

    type: "str" = betterproto2.field(4, betterproto2.TYPE_STRING)

    dest: "str" = betterproto2.field(5, betterproto2.TYPE_STRING)

    xver: "int" = betterproto2.field(6, betterproto2.TYPE_UINT64)


default_message_pool.register_message("xray.proxy.vless.inbound", "Fallback", Fallback)


from ....common import protocol as ___common__protocol__



import ipaddress
from enum import StrEnum

from pydantic import Field, field_validator, model_validator

from ._base import XBase


class QueryStrategy(StrEnum):
    use_ip = "UseIP"
    use_ip_v4 = "UseIPv4"
    use_ip_v6 = "UseIPv6"


class DnsServerObject(XBase):
    tag: str
    address: str
    port: int | None = None
    domains: list[str] | None = None
    expect_ips: list[str] | None = Field(None, serialization_alias="expectIPs")
    skip_fallback: bool = Field(False, serialization_alias="skipFallback")
    client_ip: str | None = Field(None, serialization_alias="clientIp")
    query_strategy: QueryStrategy | None = Field(None, serialization_alias="queryStrategy")
    timeout_ms: int = Field(4000, serialization_alias="timeoutMs")
    allow_unexpected_ips: bool = Field(False, serialization_alias="allowUnexpectedIPs")

    @field_validator("address")
    def validate_address(cls, v: str) -> str:
        if not v:
            raise ValueError("address cannot be empty")

        if v.startswith(("tcp://", "tcp+local://", "https://", "https+local://",
                         "h2c://", "quic+local://")):
            return v

        if v in ("fakedns", "localhost"):
            return v

        # Assume IP or domain name
        try:
            host, port_part = v.rsplit(":", 1)
            ipaddress.ip_address(host)
            return v
        except ValueError:
            # Not an IP, assume it's a domain
            return v

    @field_validator("domains")
    def validate_domains(cls, v: list[str] | None) -> list[str] | None:
        if v is None:
            return v

        for domain in v:
            if not domain.startswith(("domain:", "regexp:", "keyword:", "geosite:")):
                raise ValueError(
                    f"Invalid domain format: {domain}. Must start with domain:, regexp:, keyword:, or geosite:")
        return v

    @field_validator("expect_ips")
    def validate_expect_ips(cls, v: list[str] | None) -> list[str] | None:
        if v is None:
            return v

        for ip in v:
            if not ip.startswith(("geoip:", "cidr:")) and not cls._is_valid_ip_or_cidr(ip):
                raise ValueError(f"Invalid expectIPs entry: {ip}. Must be geoip:*, cidr:*, or a valid IP/CIDR")
        return v

    @field_validator("client_ip")
    def validate_client_ip(cls, v: str | None) -> str | None:
        if v is None:
            return v
        try:
            ip = ipaddress.ip_address(v)
            if ip.is_private:
                raise ValueError("clientIp must be a public IP address")
        except ValueError as e:
            raise ValueError(f"Invalid clientIp: {e}")
        return v

    @staticmethod
    def _is_valid_ip_or_cidr(value: str) -> bool:
        try:
            if "/" in value:
                ipaddress.ip_network(value, strict=False)
            else:
                ipaddress.ip_address(value)
            return True
        except ValueError:
            return False


class XDns(XBase):
    hosts: dict[str, list[str] | str] | None = None
    servers: list[str | DnsServerObject] | None = None
    client_ip: str | None = Field(None, serialization_alias="clientIp")
    query_strategy: QueryStrategy | None = Field(None, serialization_alias="queryStrategy")
    disable_cache: bool = Field(False, serialization_alias="disableCache")
    disable_fallback: bool = Field(False, serialization_alias="disableFallback")
    disable_fallback_if_match: bool = Field(False, serialization_alias="disableFallbackIfMatch")
    tag: str

    @field_validator("client_ip")
    def validate_client_ip_global(cls, v: str | None) -> str | None:
        if v is None:
            return v
        try:
            ip = ipaddress.ip_address(v)
            if ip.is_private:
                raise ValueError("clientIp must be a public IP address")
        except ValueError as e:
            raise ValueError(f"Invalid clientIp: {e}")
        return v

    @model_validator(mode="after")
    def validate_query_strategy_conflicts(self) -> "XDns":
        servers = self.servers
        strategy = self.query_strategy

        if not servers or strategy is None:
            return self

        for server in servers:
            if isinstance(server, DnsServerObject) and server.expect_ips:
                server_strategy = server.query_strategy
                if (
                        (strategy == QueryStrategy.use_ip_v4 and server_strategy == QueryStrategy.use_ip_v6) or
                        (strategy == QueryStrategy.use_ip_v6 and server_strategy == QueryStrategy.use_ip_v4)
                ):
                    raise ValueError(
                        f"Conflict between global queryStrategy '{strategy}' and server-specific '{server_strategy}'"
                    )
        return self



from pydantic import Field

from .._base import XBase


class XWebSocketSettings(XBase):
    accept_proxy_protocol: bool = Field(False, serialization_alias="acceptProxyProtocol")
    path: str = "/"
    host: str | None = None
    headers: dict[str, str] | None = None
    heartbeat_period: int = Field(0, serialization_alias="heartbeatPeriod")



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proxy/vmess/account.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = ("Account",)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Account(betterproto2.Message):
    id: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)
    """
    ID of the account, in the form of a UUID, e.g.,
    "66ad4540-b58c-4ad2-9926-ea63445a9b57".
    """

    security_settings: "__common__protocol__.SecurityConfig | None" = (
        betterproto2.field(3, betterproto2.TYPE_MESSAGE, optional=True)
    )
    """
    Security settings. Only applies to client side.
    """

    tests_enabled: "str" = betterproto2.field(4, betterproto2.TYPE_STRING)
    """
    Define tests enabled for this account
    """


default_message_pool.register_message("xray.proxy.vmess", "Account", Account)


from ...common import protocol as __common__protocol__



from enum import StrEnum

from .._base import XBase


class Flow(StrEnum):
    xtls = "xtls-rprx-vision"
    none = ""


class VlessClient(XBase):
    id: str
    level: int = 0
    email: str
    flow: Flow = Flow.xtls


class XVlessIn(XBase):
    clients: list[VlessClient]
    decryption: str
    fallbacks: ...



from enum import StrEnum

from pydantic import Field

from .._base import XBase


class KCPHeaderType(StrEnum):
    none = "none"
    srtp = "srtp"
    utp = "utp"
    wechat_video = "wechat-video"
    dtls = "dtls"
    wireguard = "wireguard"
    dns = "dns"


class KCPHeader(XBase):
    type: KCPHeaderType = KCPHeaderType.none
    domain: str | None = None


class XKCPSettings(XBase):
    mtu: int = Field(1350, ge=1460, le=576)
    tti: int = Field(50, ge=100, le=10)
    uplink_capacity: int = Field(5, serialization_alias="uplinkCapacity")
    downlink_capacity: int = Field(20, serialization_alias="downlinkCapacity")
    congestion: bool = False
    read_buffer_size: int = Field(2, serialization_alias="readBufferSize")
    write_buffer_size: int = Field(2, serialization_alias="writeBufferSize")

    headers: KCPHeader | None = None
    seed: str | None = None

    service_name: str = Field("/", serialization_alias="serviceName")
    multi_mode: bool = Field(False, serialization_alias="multiMode")



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: transport/internet/reality/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = ("Config",)

from dataclasses import dataclass

import betterproto2

from .....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    show: "bool" = betterproto2.field(1, betterproto2.TYPE_BOOL)

    dest: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)

    type: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)

    xver: "int" = betterproto2.field(4, betterproto2.TYPE_UINT64)

    server_names: "list[str]" = betterproto2.field(
        5, betterproto2.TYPE_STRING, repeated=True
    )

    private_key: "bytes" = betterproto2.field(6, betterproto2.TYPE_BYTES)

    min_client_ver: "bytes" = betterproto2.field(7, betterproto2.TYPE_BYTES)

    max_client_ver: "bytes" = betterproto2.field(8, betterproto2.TYPE_BYTES)

    max_time_diff: "int" = betterproto2.field(9, betterproto2.TYPE_UINT64)

    short_ids: "list[bytes]" = betterproto2.field(
        10, betterproto2.TYPE_BYTES, repeated=True
    )

    fingerprint: "str" = betterproto2.field(21, betterproto2.TYPE_STRING)

    server_name: "str" = betterproto2.field(22, betterproto2.TYPE_STRING)

    public_key: "bytes" = betterproto2.field(23, betterproto2.TYPE_BYTES)

    short_id: "bytes" = betterproto2.field(24, betterproto2.TYPE_BYTES)

    spider_x: "str" = betterproto2.field(25, betterproto2.TYPE_STRING)

    spider_y: "list[int]" = betterproto2.field(
        26, betterproto2.TYPE_INT64, repeated=True
    )

    master_key_log: "str" = betterproto2.field(27, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.transport.internet.reality", "Config", Config
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proxy/dns/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = ("Config",)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    server: "__common__net__.Endpoint | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Server is the DNS server address. If specified, this address overrides the
    original one.
    """

    user_level: "int" = betterproto2.field(2, betterproto2.TYPE_UINT32)

    non_IP_query: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)

    block_types: "list[int]" = betterproto2.field(
        4, betterproto2.TYPE_INT32, repeated=True
    )


default_message_pool.register_message("xray.proxy.dns", "Config", Config)


from ...common import net as __common__net__



from pydantic import Field

from ._base import XBase


class XObservatory(XBase):
    subject_selector: list[str] = Field(serialization_alias="subjectSelector")
    probe_url: str = Field(serialization_alias="probeUrl")
    probe_interval: str = Field("100s", serialization_alias="probeInterval")
    enable_concurrency: bool = Field(False, serialization_alias="enableConcurrency")


class PingConfig(XBase):
    destination: str
    connectivity: str = ""
    interval: str = "1h"
    sampling: int = 5
    timeout: str = "30s"


class XBurstObservatory(XBase):
    subject_selector: list[str] = Field(serialization_alias="subjectSelector")
    ping_config: PingConfig = Field(serialization_alias="pingConfig")



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/router/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "ConfigDomainStrategy",
    "DomainType",
    "BalancingRule",
    "Cidr",
    "Config",
    "Domain",
    "DomainAttribute",
    "GeoIp",
    "GeoIpList",
    "GeoSite",
    "GeoSiteList",
    "RoutingRule",
    "StrategyLeastLoadConfig",
    "StrategyWeight",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class ConfigDomainStrategy(betterproto2.Enum):
    AsIs = 0
    """
    Use domain as is.
    """

    UseIp = 1
    """
    Always resolve IP for domains.
    """

    IpIfNonMatch = 2
    """
    Resolve to IP if the domain doesn't match any rules.
    """

    IpOnDemand = 3
    """
    Resolve to IP if any rule requires IP matching.
    """


class DomainType(betterproto2.Enum):
    """
    Type of domain value.
    """

    Plain = 0
    """
    The value is used as is.
    """

    Regex = 1
    """
    The value is used as a regular expression.
    """

    Domain = 2
    """
    The value is a root domain.
    """

    Full = 3
    """
    The value is a domain.
    """


@dataclass(eq=False, repr=False)
class BalancingRule(betterproto2.Message):
    tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    outbound_selector: "list[str]" = betterproto2.field(
        2, betterproto2.TYPE_STRING, repeated=True
    )

    strategy: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)

    strategy_settings: "__common__serial__.TypedMessage | None" = betterproto2.field(
        4, betterproto2.TYPE_MESSAGE, optional=True
    )

    fallback_tag: "str" = betterproto2.field(5, betterproto2.TYPE_STRING)


default_message_pool.register_message("xray.app.router", "BalancingRule", BalancingRule)


@dataclass(eq=False, repr=False)
class Cidr(betterproto2.Message):
    """
    IP for routing decision, in CIDR form.
    """

    ip: "bytes" = betterproto2.field(1, betterproto2.TYPE_BYTES)
    """
    IP address, should be either 4 or 16 bytes.
    """

    prefix: "int" = betterproto2.field(2, betterproto2.TYPE_UINT32)
    """
    Number of leading ones in the network mask.
    """


default_message_pool.register_message("xray.app.router", "CIDR", Cidr)


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    domain_strategy: "ConfigDomainStrategy" = betterproto2.field(
        1, betterproto2.TYPE_ENUM, default_factory=lambda: ConfigDomainStrategy(0)
    )

    rule: "list[RoutingRule]" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, repeated=True
    )

    balancing_rule: "list[BalancingRule]" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message("xray.app.router", "Config", Config)


@dataclass(eq=False, repr=False)
class Domain(betterproto2.Message):
    """
    Domain for routing decision.
    """

    type: "DomainType" = betterproto2.field(
        1, betterproto2.TYPE_ENUM, default_factory=lambda: DomainType(0)
    )
    """
    Domain matching type.
    """

    value: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)
    """
    Domain value.
    """

    attribute: "list[DomainAttribute]" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, repeated=True
    )
    """
    Attributes of this domain. May be used for filtering.
    """


default_message_pool.register_message("xray.app.router", "Domain", Domain)


@dataclass(eq=False, repr=False)
class DomainAttribute(betterproto2.Message):
    """


    Oneofs:
        - typed_value:
    """

    key: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    bool_value: "bool | None" = betterproto2.field(
        2, betterproto2.TYPE_BOOL, optional=True, group="typed_value"
    )

    int_value: "int | None" = betterproto2.field(
        3, betterproto2.TYPE_INT64, optional=True, group="typed_value"
    )


default_message_pool.register_message(
    "xray.app.router", "Domain.Attribute", DomainAttribute
)


@dataclass(eq=False, repr=False)
class GeoIp(betterproto2.Message):
    country_code: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    cidr: "list[Cidr]" = betterproto2.field(2, betterproto2.TYPE_MESSAGE, repeated=True)

    reverse_match: "bool" = betterproto2.field(3, betterproto2.TYPE_BOOL)


default_message_pool.register_message("xray.app.router", "GeoIP", GeoIp)


@dataclass(eq=False, repr=False)
class GeoIpList(betterproto2.Message):
    entry: "list[GeoIp]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message("xray.app.router", "GeoIPList", GeoIpList)


@dataclass(eq=False, repr=False)
class GeoSite(betterproto2.Message):
    country_code: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    domain: "list[Domain]" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message("xray.app.router", "GeoSite", GeoSite)


@dataclass(eq=False, repr=False)
class GeoSiteList(betterproto2.Message):
    entry: "list[GeoSite]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message("xray.app.router", "GeoSiteList", GeoSiteList)


@dataclass(eq=False, repr=False)
class RoutingRule(betterproto2.Message):
    """


    Oneofs:
        - target_tag:
    """

    tag: "str | None" = betterproto2.field(
        1, betterproto2.TYPE_STRING, optional=True, group="target_tag"
    )
    """
    Tag of outbound that this rule is pointing to.
    """

    balancing_tag: "str | None" = betterproto2.field(
        12, betterproto2.TYPE_STRING, optional=True, group="target_tag"
    )
    """
    Tag of routing balancer.
    """

    rule_tag: "str" = betterproto2.field(18, betterproto2.TYPE_STRING)

    domain: "list[Domain]" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, repeated=True
    )
    """
    List of domains for target domain matching.
    """

    geoip: "list[GeoIp]" = betterproto2.field(
        10, betterproto2.TYPE_MESSAGE, repeated=True
    )
    """
    List of GeoIPs for target IP address matching. If this entry exists, the
    cidr above will have no effect. GeoIP fields with the same country code are
    supposed to contain exactly same content. They will be merged during
    runtime. For customized GeoIPs, please leave country code empty.
    """

    port_list: "__common__net__.PortList | None" = betterproto2.field(
        14, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    List of ports.
    """

    networks: "list[__common__net__.Network]" = betterproto2.field(
        13, betterproto2.TYPE_ENUM, repeated=True
    )
    """
    List of networks for matching.
    """

    source_geoip: "list[GeoIp]" = betterproto2.field(
        11, betterproto2.TYPE_MESSAGE, repeated=True
    )
    """
    List of GeoIPs for source IP address matching. If this entry exists, the
    source_cidr above will have no effect.
    """

    source_port_list: "__common__net__.PortList | None" = betterproto2.field(
        16, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    List of ports for source port matching.
    """

    user_email: "list[str]" = betterproto2.field(
        7, betterproto2.TYPE_STRING, repeated=True
    )

    inbound_tag: "list[str]" = betterproto2.field(
        8, betterproto2.TYPE_STRING, repeated=True
    )

    protocol: "list[str]" = betterproto2.field(
        9, betterproto2.TYPE_STRING, repeated=True
    )

    attributes: "dict[str, str]" = betterproto2.field(
        15,
        betterproto2.TYPE_MAP,
        map_meta=betterproto2.map_meta(
            betterproto2.TYPE_STRING, betterproto2.TYPE_STRING
        ),
    )

    domain_matcher: "str" = betterproto2.field(17, betterproto2.TYPE_STRING)


default_message_pool.register_message("xray.app.router", "RoutingRule", RoutingRule)


@dataclass(eq=False, repr=False)
class StrategyLeastLoadConfig(betterproto2.Message):
    costs: "list[StrategyWeight]" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, repeated=True
    )
    """
    weight settings
    """

    baselines: "list[int]" = betterproto2.field(
        3, betterproto2.TYPE_INT64, repeated=True
    )
    """
    RTT baselines for selecting, int64 values of time.Duration
    """

    expected: "int" = betterproto2.field(4, betterproto2.TYPE_INT32)
    """
    expected nodes count to select
    """

    max_rtt: "int" = betterproto2.field(5, betterproto2.TYPE_INT64)
    """
    max acceptable rtt, filter away high delay nodes. default 0
    """

    tolerance: "float" = betterproto2.field(6, betterproto2.TYPE_FLOAT)
    """
    acceptable failure rate
    """


default_message_pool.register_message(
    "xray.app.router", "StrategyLeastLoadConfig", StrategyLeastLoadConfig
)


@dataclass(eq=False, repr=False)
class StrategyWeight(betterproto2.Message):
    regexp: "bool" = betterproto2.field(1, betterproto2.TYPE_BOOL)

    match: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)

    value: "float" = betterproto2.field(3, betterproto2.TYPE_FLOAT)


default_message_pool.register_message(
    "xray.app.router", "StrategyWeight", StrategyWeight
)


from ...common import net as __common__net__
from ...common import serial as __common__serial__



from grpclib.exceptions import GRPCError

from xray_cfo.xray.app.proxyman.command import RemoveInboundRequest, RemoveOutboundRequest

from .._base import BaseService


class HandlerRemoveBoundService(BaseService):

    async def remove_inbound(self, tag: str):
        try:
            await self.handler_stub.remove_inbound(RemoveInboundRequest(tag=tag))
            return True
        except GRPCError:
            return None

    async def remove_outbound(self, tag: str):
        try:
            await self.handler_stub.remove_outbound(RemoveOutboundRequest(tag=tag))
            return True
        except GRPCError:
            return None



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/commander/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "ReflectionConfig",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    """
    Config is the settings for Commander.
    """

    tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)
    """
    Tag of the outbound handler that handles grpc connections.
    """

    listen: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)
    """
    Network address of commander grpc service.
    """

    service: "list[__common__serial__.TypedMessage]" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, repeated=True
    )
    """
    Services that supported by this server. All services must implement Service
    interface.
    """


default_message_pool.register_message("xray.app.commander", "Config", Config)


@dataclass(eq=False, repr=False)
class ReflectionConfig(betterproto2.Message):
    """
    ReflectionConfig is the placeholder config for ReflectionService.
    """

    pass


default_message_pool.register_message(
    "xray.app.commander", "ReflectionConfig", ReflectionConfig
)


from ...common import serial as __common__serial__



from ._base import XBase


class Bridge(XBase):
    tag: str
    domain: str


class Portal(XBase):
    tag: str
    domain: str


class XReverse(XBase):
    bridges: list[Bridge] | None = None
    portals: list[Portal] | None = None



from xray_cfo.api_services.handler.add_client import HandlerAddClientService
from xray_cfo.api_services.handler.add_inbound import HandlerAddInboundService
from xray_cfo.api_services.handler.add_outbound import HandlerAddOutboundService
from xray_cfo.api_services.handler.get_inbound_users import HandlerGetInboundUsersService
from xray_cfo.api_services.handler.list_bound import HandlerListBoundService
from xray_cfo.api_services.handler.remove_bound import HandlerRemoveBoundService
from xray_cfo.api_services.handler.remove_client import HandlerRemoveClientService


class HandlerService(
    HandlerGetInboundUsersService,
    HandlerAddClientService,
    HandlerListBoundService,
    HandlerRemoveBoundService,
    HandlerAddOutboundService,
    HandlerAddInboundService,
    HandlerRemoveClientService,
): ...



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proxy/vless/account.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = ("Account",)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Account(betterproto2.Message):
    id: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)
    """
    ID of the account, in the form of a UUID, e.g., "66ad4540-b58c-4ad2-9926-ea63445a9b57".
    """

    flow: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)
    """
    Flow settings. May be "xtls-rprx-vision".
    """

    encryption: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)
    """
    Encryption settings. Only applies to client side, and only accepts "none" for now.
    """


default_message_pool.register_message("xray.proxy.vless", "Account", Account)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: transport/internet/grpc/config.proto, transport/internet/grpc/encoding/stream.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "Hunk",
    "MultiHunk",
    "GrpcServiceStub",
    "GrpcServiceBase",
)

from collections.abc import AsyncIterable, AsyncIterator, Iterable
from dataclasses import dataclass
from typing import TYPE_CHECKING

import betterproto2
import grpclib
from betterproto2.grpc.grpclib_server import ServiceBase

from ......message_pool import default_message_pool

if TYPE_CHECKING:
    import grpclib.server
    from betterproto2.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    authority: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    service_name: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)

    multi_mode: "bool" = betterproto2.field(3, betterproto2.TYPE_BOOL)

    idle_timeout: "int" = betterproto2.field(4, betterproto2.TYPE_INT32)

    health_check_timeout: "int" = betterproto2.field(5, betterproto2.TYPE_INT32)

    permit_without_stream: "bool" = betterproto2.field(6, betterproto2.TYPE_BOOL)

    initial_windows_size: "int" = betterproto2.field(7, betterproto2.TYPE_INT32)

    user_agent: "str" = betterproto2.field(8, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.transport.internet.grpc.encoding", "Config", Config
)


@dataclass(eq=False, repr=False)
class Hunk(betterproto2.Message):
    data: "bytes" = betterproto2.field(1, betterproto2.TYPE_BYTES)


default_message_pool.register_message(
    "xray.transport.internet.grpc.encoding", "Hunk", Hunk
)


@dataclass(eq=False, repr=False)
class MultiHunk(betterproto2.Message):
    data: "list[bytes]" = betterproto2.field(1, betterproto2.TYPE_BYTES, repeated=True)


default_message_pool.register_message(
    "xray.transport.internet.grpc.encoding", "MultiHunk", MultiHunk
)


class GrpcServiceStub(betterproto2.ServiceStub):
    async def tun(
        self,
        messages: "AsyncIterable[Hunk] | Iterable[Hunk]",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "AsyncIterator[Hunk]":
        async for response in self._stream_stream(
            "/xray.transport.internet.grpc.encoding.GRPCService/Tun",
            messages,
            Hunk,
            Hunk,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    async def tun_multi(
        self,
        messages: "AsyncIterable[MultiHunk] | Iterable[MultiHunk]",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "AsyncIterator[MultiHunk]":
        async for response in self._stream_stream(
            "/xray.transport.internet.grpc.encoding.GRPCService/TunMulti",
            messages,
            MultiHunk,
            MultiHunk,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response


class GrpcServiceBase(ServiceBase):
    async def tun(self, messages: "AsyncIterator[Hunk]") -> "AsyncIterator[Hunk]":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def tun_multi(
        self, messages: "AsyncIterator[MultiHunk]"
    ) -> "AsyncIterator[MultiHunk]":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_tun(self, stream: "grpclib.server.Stream[Hunk, Hunk]") -> None:
        request = stream.__aiter__()
        await self._call_rpc_handler_server_stream(
            self.tun,
            stream,
            request,
        )

    async def __rpc_tun_multi(
        self, stream: "grpclib.server.Stream[MultiHunk, MultiHunk]"
    ) -> None:
        request = stream.__aiter__()
        await self._call_rpc_handler_server_stream(
            self.tun_multi,
            stream,
            request,
        )

    def __mapping__(self) -> "dict[str, grpclib.const.Handler]":
        return {
            "/xray.transport.internet.grpc.encoding.GRPCService/Tun": grpclib.const.Handler(
                self.__rpc_tun,
                grpclib.const.Cardinality.STREAM_STREAM,
                Hunk,
                Hunk,
            ),
            "/xray.transport.internet.grpc.encoding.GRPCService/TunMulti": grpclib.const.Handler(
                self.__rpc_tun_multi,
                grpclib.const.Cardinality.STREAM_STREAM,
                MultiHunk,
                MultiHunk,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: transport/internet/httpupgrade/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = ("Config",)

from dataclasses import dataclass

import betterproto2

from .....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    host: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    path: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)

    header: "dict[str, str]" = betterproto2.field(
        3,
        betterproto2.TYPE_MAP,
        map_meta=betterproto2.map_meta(
            betterproto2.TYPE_STRING, betterproto2.TYPE_STRING
        ),
    )

    accept_proxy_protocol: "bool" = betterproto2.field(4, betterproto2.TYPE_BOOL)

    ed: "int" = betterproto2.field(5, betterproto2.TYPE_UINT32)


default_message_pool.register_message(
    "xray.transport.internet.httpupgrade", "Config", Config
)



from pydantic import Field

from .._base import XBase


class RawNoneHeader(XBase):
    type: str = "none"


def base_path_list():
    return ["/"]


class RawHttpRequest(XBase):
    version: str = "1.1"
    method: str = "GET"
    path: list[str] = Field(default_factory=base_path_list)
    headers: dict[str, list[str] | str]


class RawHttpResponse(XBase):
    version: str = "1.1"
    status: str = "200"
    reason: str = "OK"
    headers: dict[str, list[str] | str]


class RawHttpHeader(XBase):
    type: str = "http"
    request: RawHttpRequest | None = None
    response: RawHttpResponse | None = None


class XRawSettings(XBase):
    accept_proxy_protocol: bool = Field(False, serialization_alias="acceptProxyProtocol")
    headers: RawNoneHeader | None = None



from enum import StrEnum

from .._base import XBase


class Flow(StrEnum):
    xtls = "xtls-rprx-vision"
    none = ""


class VmessClient(XBase):
    id: str
    level: int = 0
    email: str


class DefaultObject(XBase):
    level: int = 0


class DetourObject(XBase):
    to: str = 0


class XVmessIn(XBase):
    clients: list[VmessClient]
    default: DefaultObject
    detour: DetourObject



from grpclib.exceptions import GRPCError

from xray_cfo.xray.app.stats.command import GetStatsRequest, QueryStatsRequest

from .._base import BaseService


class StatsAPIService(BaseService):

    async def get_client_download_traffic(self, email: str, reset: bool = False):
        try:
            return (
                await self.stats_stub.get_stats(
                    GetStatsRequest(
                        name=f"user>>>{email}>>>traffic>>>downlink", reset=reset
                    )
                )
            ).stat.value
        except GRPCError:
            return None

    async def get_client_upload_traffic(self, email: str, reset=False):
        try:
            return (
                await self.stats_stub.get_stats(
                    GetStatsRequest(
                        name=f"user>>>{email}>>>traffic>>>uplink", reset=reset
                    )
                )
            ).stat.value
        except GRPCError:
            return None

    async def get_inbound_download_traffic(self, tag: str, reset=False):
        try:
            return (
                await self.stats_stub.get_stats(
                    GetStatsRequest(
                        name=f"inbound>>>{tag}>>>traffic>>>downlink", reset=reset
                    )
                )
            ).stat.value
        except GRPCError:
            return None

    async def get_inbound_upload_traffic(self, tag: str, reset=False):
        try:
            return (
                await self.stats_stub.get_stats(
                    GetStatsRequest(
                        name=f"inbound>>>{tag}>>>traffic>>>uplink", reset=reset
                    )
                )
            ).stat.value
        except GRPCError:
            return None

    async def stats_online(self, user: str, reset: bool = False):
        try:
            return (
                await self.stats_stub.get_stats_online(
                    GetStatsRequest(name=f"user>>>{user}>>>online", reset=reset)
                )
            ).stat.value
        except GRPCError:
            return None

    async def stats_query(self, pattern: str, reset: bool = False):
        try:
            return (
                await self.stats_stub.query_stats(
                    QueryStatsRequest(pattern=pattern, reset=reset)
                )
            ).stat.value
        except GRPCError:
            return None

    async def get_sys_stats(self):
        try:
            sys_stats = await self.stats_stub.get_sys_stats()

            # TODO: add Model to return

            return sys_stats
        except GRPCError:
            return None

    async def get_stats_online_ip_list(self, name: str, reset: bool = False):
        try:
            return (
                await self.stats_stub.get_stats_online_ip_list(
                    GetStatsRequest(name=name, reset=reset)
                )
            ).ips
        except GRPCError:
            return None



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/stats/command/command.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "GetStatsOnlineIpListResponse",
    "GetStatsRequest",
    "GetStatsResponse",
    "QueryStatsRequest",
    "QueryStatsResponse",
    "Stat",
    "SysStatsRequest",
    "SysStatsResponse",
    "StatsServiceStub",
    "StatsServiceBase",
)

from dataclasses import dataclass
from typing import TYPE_CHECKING

import betterproto2
import grpclib
from betterproto2.grpc.grpclib_server import ServiceBase

from .....message_pool import default_message_pool

if TYPE_CHECKING:
    import grpclib.server
    from betterproto2.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    pass


default_message_pool.register_message("xray.app.stats.command", "Config", Config)


@dataclass(eq=False, repr=False)
class GetStatsOnlineIpListResponse(betterproto2.Message):
    name: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    ips: "dict[str, int]" = betterproto2.field(
        2,
        betterproto2.TYPE_MAP,
        map_meta=betterproto2.map_meta(
            betterproto2.TYPE_STRING, betterproto2.TYPE_INT64
        ),
    )


default_message_pool.register_message(
    "xray.app.stats.command",
    "GetStatsOnlineIpListResponse",
    GetStatsOnlineIpListResponse,
)


@dataclass(eq=False, repr=False)
class GetStatsRequest(betterproto2.Message):
    name: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)
    """
    Name of the stat counter.
    """

    reset: "bool" = betterproto2.field(2, betterproto2.TYPE_BOOL)
    """
    Whether or not to reset the counter to fetching its value.
    """


default_message_pool.register_message(
    "xray.app.stats.command", "GetStatsRequest", GetStatsRequest
)


@dataclass(eq=False, repr=False)
class GetStatsResponse(betterproto2.Message):
    stat: "Stat | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.app.stats.command", "GetStatsResponse", GetStatsResponse
)


@dataclass(eq=False, repr=False)
class QueryStatsRequest(betterproto2.Message):
    pattern: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    reset: "bool" = betterproto2.field(2, betterproto2.TYPE_BOOL)


default_message_pool.register_message(
    "xray.app.stats.command", "QueryStatsRequest", QueryStatsRequest
)


@dataclass(eq=False, repr=False)
class QueryStatsResponse(betterproto2.Message):
    stat: "list[Stat]" = betterproto2.field(1, betterproto2.TYPE_MESSAGE, repeated=True)


default_message_pool.register_message(
    "xray.app.stats.command", "QueryStatsResponse", QueryStatsResponse
)


@dataclass(eq=False, repr=False)
class Stat(betterproto2.Message):
    name: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    value: "int" = betterproto2.field(2, betterproto2.TYPE_INT64)


default_message_pool.register_message("xray.app.stats.command", "Stat", Stat)


@dataclass(eq=False, repr=False)
class SysStatsRequest(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.stats.command", "SysStatsRequest", SysStatsRequest
)


@dataclass(eq=False, repr=False)
class SysStatsResponse(betterproto2.Message):
    num_goroutine: "int" = betterproto2.field(1, betterproto2.TYPE_UINT32)

    num_gc: "int" = betterproto2.field(2, betterproto2.TYPE_UINT32)

    alloc: "int" = betterproto2.field(3, betterproto2.TYPE_UINT64)

    total_alloc: "int" = betterproto2.field(4, betterproto2.TYPE_UINT64)

    sys: "int" = betterproto2.field(5, betterproto2.TYPE_UINT64)

    mallocs: "int" = betterproto2.field(6, betterproto2.TYPE_UINT64)

    frees: "int" = betterproto2.field(7, betterproto2.TYPE_UINT64)

    live_objects: "int" = betterproto2.field(8, betterproto2.TYPE_UINT64)

    pause_total_ns: "int" = betterproto2.field(9, betterproto2.TYPE_UINT64)

    uptime: "int" = betterproto2.field(10, betterproto2.TYPE_UINT32)


default_message_pool.register_message(
    "xray.app.stats.command", "SysStatsResponse", SysStatsResponse
)


class StatsServiceStub(betterproto2.ServiceStub):
    async def get_stats(
        self,
        message: "GetStatsRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "GetStatsResponse":
        return await self._unary_unary(
            "/xray.app.stats.command.StatsService/GetStats",
            message,
            GetStatsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_stats_online(
        self,
        message: "GetStatsRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "GetStatsResponse":
        return await self._unary_unary(
            "/xray.app.stats.command.StatsService/GetStatsOnline",
            message,
            GetStatsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def query_stats(
        self,
        message: "QueryStatsRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "QueryStatsResponse":
        return await self._unary_unary(
            "/xray.app.stats.command.StatsService/QueryStats",
            message,
            QueryStatsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_sys_stats(
        self,
        message: "SysStatsRequest | None" = None,
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "SysStatsResponse":
        if message is None:
            message = SysStatsRequest()

        return await self._unary_unary(
            "/xray.app.stats.command.StatsService/GetSysStats",
            message,
            SysStatsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_stats_online_ip_list(
        self,
        message: "GetStatsRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "GetStatsOnlineIpListResponse":
        return await self._unary_unary(
            "/xray.app.stats.command.StatsService/GetStatsOnlineIpList",
            message,
            GetStatsOnlineIpListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class StatsServiceBase(ServiceBase):
    async def get_stats(self, message: "GetStatsRequest") -> "GetStatsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_stats_online(self, message: "GetStatsRequest") -> "GetStatsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def query_stats(self, message: "QueryStatsRequest") -> "QueryStatsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_sys_stats(self, message: "SysStatsRequest") -> "SysStatsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_stats_online_ip_list(
        self, message: "GetStatsRequest"
    ) -> "GetStatsOnlineIpListResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_stats(
        self, stream: "grpclib.server.Stream[GetStatsRequest, GetStatsResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_stats(request)
        await stream.send_message(response)

    async def __rpc_get_stats_online(
        self, stream: "grpclib.server.Stream[GetStatsRequest, GetStatsResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_stats_online(request)
        await stream.send_message(response)

    async def __rpc_query_stats(
        self, stream: "grpclib.server.Stream[QueryStatsRequest, QueryStatsResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.query_stats(request)
        await stream.send_message(response)

    async def __rpc_get_sys_stats(
        self, stream: "grpclib.server.Stream[SysStatsRequest, SysStatsResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_sys_stats(request)
        await stream.send_message(response)

    async def __rpc_get_stats_online_ip_list(
        self,
        stream: "grpclib.server.Stream[GetStatsRequest, GetStatsOnlineIpListResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_stats_online_ip_list(request)
        await stream.send_message(response)

    def __mapping__(self) -> "dict[str, grpclib.const.Handler]":
        return {
            "/xray.app.stats.command.StatsService/GetStats": grpclib.const.Handler(
                self.__rpc_get_stats,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetStatsRequest,
                GetStatsResponse,
            ),
            "/xray.app.stats.command.StatsService/GetStatsOnline": grpclib.const.Handler(
                self.__rpc_get_stats_online,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetStatsRequest,
                GetStatsResponse,
            ),
            "/xray.app.stats.command.StatsService/QueryStats": grpclib.const.Handler(
                self.__rpc_query_stats,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryStatsRequest,
                QueryStatsResponse,
            ),
            "/xray.app.stats.command.StatsService/GetSysStats": grpclib.const.Handler(
                self.__rpc_get_sys_stats,
                grpclib.const.Cardinality.UNARY_UNARY,
                SysStatsRequest,
                SysStatsResponse,
            ),
            "/xray.app.stats.command.StatsService/GetStatsOnlineIpList": grpclib.const.Handler(
                self.__rpc_get_stats_online_ip_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetStatsRequest,
                GetStatsOnlineIpListResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/dispatcher/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "SessionConfig",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    settings: "SessionConfig | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message("xray.app.dispatcher", "Config", Config)


@dataclass(eq=False, repr=False)
class SessionConfig(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.dispatcher", "SessionConfig", SessionConfig
)



from grpclib.exceptions import GRPCError

from .._base import BaseService


class ObservatoryService(BaseService):

    async def get_outbound_status(self):

        try:
            return (await self.observatory_stub.get_outbound_status()).status.status
        except GRPCError:
            return None



from pydantic import Field

from .._base import XBase


class Peer(XBase):
    allowed_ips: list[str] = Field(serialization_alias="allowedIPs")
    keep_alive: int
    pre_shared_key: str = Field(serialization_alias="preSharedKey")
    private_key: str = Field(serialization_alias="publicKey")
    public_key: str = Field(serialization_alias="publicKey")


class WireGuardIn(XBase):
    secret_key: str = Field(serialization_alias="secretKey")
    peers: list[Peer]
    mtu: int = 1420



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/reverse/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "ControlState",
    "BridgeConfig",
    "Config",
    "Control",
    "PortalConfig",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class ControlState(betterproto2.Enum):
    ACTIVE = 0

    DRAIN = 1


@dataclass(eq=False, repr=False)
class BridgeConfig(betterproto2.Message):
    tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    domain: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)


default_message_pool.register_message("xray.app.reverse", "BridgeConfig", BridgeConfig)


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    bridge_config: "list[BridgeConfig]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )

    portal_config: "list[PortalConfig]" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message("xray.app.reverse", "Config", Config)


@dataclass(eq=False, repr=False)
class Control(betterproto2.Message):
    state: "ControlState" = betterproto2.field(
        1, betterproto2.TYPE_ENUM, default_factory=lambda: ControlState(0)
    )

    random: "bytes" = betterproto2.field(99, betterproto2.TYPE_BYTES)


default_message_pool.register_message("xray.app.reverse", "Control", Control)


@dataclass(eq=False, repr=False)
class PortalConfig(betterproto2.Message):
    tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    domain: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)


default_message_pool.register_message("xray.app.reverse", "PortalConfig", PortalConfig)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proxy/shadowsocks/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "CipherType",
    "Account",
    "ClientConfig",
    "ServerConfig",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class CipherType(betterproto2.Enum):
    UNKNOWN = 0

    AES_128_GCM = 5

    AES_256_GCM = 6

    CHACHA20_POLY1305 = 7

    XCHACHA20_POLY1305 = 8

    NONE = 9


@dataclass(eq=False, repr=False)
class Account(betterproto2.Message):
    password: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    cipher_type: "CipherType" = betterproto2.field(
        2, betterproto2.TYPE_ENUM, default_factory=lambda: CipherType(0)
    )

    iv_check: "bool" = betterproto2.field(3, betterproto2.TYPE_BOOL)


default_message_pool.register_message("xray.proxy.shadowsocks", "Account", Account)


@dataclass(eq=False, repr=False)
class ClientConfig(betterproto2.Message):
    server: "list[__common__protocol__.ServerEndpoint]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message(
    "xray.proxy.shadowsocks", "ClientConfig", ClientConfig
)


@dataclass(eq=False, repr=False)
class ServerConfig(betterproto2.Message):
    users: "list[__common__protocol__.User]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )

    network: "list[__common__net__.Network]" = betterproto2.field(
        2, betterproto2.TYPE_ENUM, repeated=True
    )


default_message_pool.register_message(
    "xray.proxy.shadowsocks", "ServerConfig", ServerConfig
)


from ...common import net as __common__net__
from ...common import protocol as __common__protocol__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/observatory/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "HealthPingMeasurementResult",
    "Intensity",
    "ObservationResult",
    "OutboundStatus",
    "ProbeResult",
)

from dataclasses import dataclass

import betterproto2

from .....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    subject_selector: "list[str]" = betterproto2.field(
        2, betterproto2.TYPE_STRING, repeated=True
    )
    """
    @Document The selectors for outbound under observation
    """

    probe_url: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)

    probe_interval: "int" = betterproto2.field(4, betterproto2.TYPE_INT64)

    enable_concurrency: "bool" = betterproto2.field(5, betterproto2.TYPE_BOOL)


default_message_pool.register_message("xray.core.app.observatory", "Config", Config)


@dataclass(eq=False, repr=False)
class HealthPingMeasurementResult(betterproto2.Message):
    all: "int" = betterproto2.field(1, betterproto2.TYPE_INT64)

    fail: "int" = betterproto2.field(2, betterproto2.TYPE_INT64)

    deviation: "int" = betterproto2.field(3, betterproto2.TYPE_INT64)

    average: "int" = betterproto2.field(4, betterproto2.TYPE_INT64)

    max: "int" = betterproto2.field(5, betterproto2.TYPE_INT64)

    min: "int" = betterproto2.field(6, betterproto2.TYPE_INT64)


default_message_pool.register_message(
    "xray.core.app.observatory",
    "HealthPingMeasurementResult",
    HealthPingMeasurementResult,
)


@dataclass(eq=False, repr=False)
class Intensity(betterproto2.Message):
    probe_interval: "int" = betterproto2.field(1, betterproto2.TYPE_UINT32)
    """
    @Document The time interval for a probe request in ms.
    @Type time.ms
    """


default_message_pool.register_message(
    "xray.core.app.observatory", "Intensity", Intensity
)


@dataclass(eq=False, repr=False)
class ObservationResult(betterproto2.Message):
    status: "list[OutboundStatus]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message(
    "xray.core.app.observatory", "ObservationResult", ObservationResult
)


@dataclass(eq=False, repr=False)
class OutboundStatus(betterproto2.Message):
    alive: "bool" = betterproto2.field(1, betterproto2.TYPE_BOOL)
    """
    @Document Whether this outbound is usable
    @Restriction ReadOnlyForUser
    """

    delay: "int" = betterproto2.field(2, betterproto2.TYPE_INT64)
    """
    @Document The time for probe request to finish.
    @Type time.ms
    @Restriction ReadOnlyForUser
    """

    last_error_reason: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)
    """
    @Document The last error caused this outbound failed to relay probe request
    @Restriction NotMachineReadable
    """

    outbound_tag: "str" = betterproto2.field(4, betterproto2.TYPE_STRING)
    """
    @Document The outbound tag for this Server
    @Type id.outboundTag
    """

    last_seen_time: "int" = betterproto2.field(5, betterproto2.TYPE_INT64)
    """
    @Document The time this outbound is known to be alive
    @Type id.outboundTag
    """

    last_try_time: "int" = betterproto2.field(6, betterproto2.TYPE_INT64)
    """
    @Document The time this outbound is tried
    @Type id.outboundTag
    """

    health_ping: "HealthPingMeasurementResult | None" = betterproto2.field(
        7, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.core.app.observatory", "OutboundStatus", OutboundStatus
)


@dataclass(eq=False, repr=False)
class ProbeResult(betterproto2.Message):
    alive: "bool" = betterproto2.field(1, betterproto2.TYPE_BOOL)
    """
    @Document Whether this outbound is usable
    @Restriction ReadOnlyForUser
    """

    delay: "int" = betterproto2.field(2, betterproto2.TYPE_INT64)
    """
    @Document The time for probe request to finish.
    @Type time.ms
    @Restriction ReadOnlyForUser
    """

    last_error_reason: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)
    """
    @Document The error caused this outbound failed to relay probe request
    @Restriction NotMachineReadable
    """


default_message_pool.register_message(
    "xray.core.app.observatory", "ProbeResult", ProbeResult
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/log/command/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "RestartLoggerRequest",
    "RestartLoggerResponse",
    "LoggerServiceStub",
    "LoggerServiceBase",
)

from dataclasses import dataclass
from typing import TYPE_CHECKING

import betterproto2
import grpclib
from betterproto2.grpc.grpclib_server import ServiceBase

from .....message_pool import default_message_pool

if TYPE_CHECKING:
    import grpclib.server
    from betterproto2.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    pass


default_message_pool.register_message("xray.app.log.command", "Config", Config)


@dataclass(eq=False, repr=False)
class RestartLoggerRequest(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.log.command", "RestartLoggerRequest", RestartLoggerRequest
)


@dataclass(eq=False, repr=False)
class RestartLoggerResponse(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.log.command", "RestartLoggerResponse", RestartLoggerResponse
)


class LoggerServiceStub(betterproto2.ServiceStub):
    async def restart_logger(
        self,
        message: "RestartLoggerRequest | None" = None,
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "RestartLoggerResponse":
        if message is None:
            message = RestartLoggerRequest()

        return await self._unary_unary(
            "/xray.app.log.command.LoggerService/RestartLogger",
            message,
            RestartLoggerResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class LoggerServiceBase(ServiceBase):
    async def restart_logger(
        self, message: "RestartLoggerRequest"
    ) -> "RestartLoggerResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_restart_logger(
        self,
        stream: "grpclib.server.Stream[RestartLoggerRequest, RestartLoggerResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.restart_logger(request)
        await stream.send_message(response)

    def __mapping__(self) -> "dict[str, grpclib.const.Handler]":
        return {
            "/xray.app.log.command.LoggerService/RestartLogger": grpclib.const.Handler(
                self.__rpc_restart_logger,
                grpclib.const.Cardinality.UNARY_UNARY,
                RestartLoggerRequest,
                RestartLoggerResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: transport/internet/tcp/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = ("Config",)

from dataclasses import dataclass

import betterproto2

from .....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    header_settings: "___common__serial__.TypedMessage | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )

    accept_proxy_protocol: "bool" = betterproto2.field(3, betterproto2.TYPE_BOOL)


default_message_pool.register_message("xray.transport.internet.tcp", "Config", Config)


from ....common import serial as ___common__serial__



from enum import StrEnum

from pydantic import BaseModel, Field

from ._base import XBase


class LogLevel(StrEnum):
    debug = "debug"
    info = "info"
    warning = "warning"
    error = "error"
    none = "none"


class MaskAddress(StrEnum):
    quarter = "quarter"
    half = "half"
    full = "full"
    none = ""


class XLog(XBase):
    access: str = ""
    error: str = ""
    loglevel: LogLevel = LogLevel.none
    dns_log: bool = Field(False, serialization_alias="dnsLog")
    mask_address: MaskAddress = MaskAddress.none



from pydantic import Field

from .._base import XBase


class HttpAccount(XBase):
    user: str
    password: str = Field(serialization_alias="pass")


class XHttpIn(XBase):
    accounts: list[HttpAccount]
    allow_transparent: bool = Field(False, serialization_alias="allowTransparent:")
    user_level: int = Field(0, serialization_alias="userLevel")



from grpclib.exceptions import GRPCError

from xray_cfo.api_services._base import BaseService
from xray_cfo.xray.app.proxyman.command import GetInboundUserRequest


class HandlerGetInboundUsersService(BaseService):

    async def get_inbound_users(self, tag: str, email: str):
        try:
            return (
                await self.handler_stub.get_inbound_users(
                    GetInboundUserRequest(tag=tag, email=email)
                )
            ).users
        except GRPCError:
            return None

    async def get_inbound_users_count(self, tag: str, email: str):
        try:
            return (
                await self.handler_stub.get_inbound_users_count(
                    GetInboundUserRequest(tag=tag, email=email)
                )
            ).count
        except GRPCError:
            return None



from pydantic import Field

from ._base import XBase


class LevelPolicy(XBase):
    handshake: int = 4
    conn_idle: int = Field(300, serialization_alias="connIdle")
    uplink_only: int = Field(2, serialization_alias="uplinkOnly")
    downlink_only: int = Field(5, serialization_alias="downlinkOnly")
    stats_user_Uplink: bool = Field(False, serialization_alias="statsUserUplink")
    stats_user_downlink: bool = Field(False, serialization_alias="statsUserDownlink")
    stats_user_online: bool = Field(False, serialization_alias="statsUserOnline")
    buffer_size: int = Field(10240, serialization_alias="statsInboundUplink")


class SystemPolicy(XBase):
    stats_inbound_uplink: bool = Field(False, serialization_alias="statsInboundUplink")
    stats_inbound_downlink: bool = Field(False, serialization_alias="statsInboundDownlink")
    stats_outbound_uplink: bool = Field(False, serialization_alias="statsOutboundUplink")
    stats_outbound_downlink: bool = Field(False, serialization_alias="statsOutboundDownlink")


class XPolicy(XBase):
    levels: dict[str, LevelPolicy]
    system: SystemPolicy



from xray_cfo.api_services.grpc import GRPCService
from xray_cfo.api_services.handler import HandlerService
from xray_cfo.api_services.logger import LoggerService
from xray_cfo.api_services.observatory import ObservatoryService
from xray_cfo.api_services.routing import RoutingService
from xray_cfo.api_services.stats import StatsAPIService


class APIService(
    StatsAPIService,
    LoggerService,
    ObservatoryService,
    GRPCService,
    RoutingService,
    HandlerService,
):
    pass



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/observatory/command/command.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "GetOutboundStatusRequest",
    "GetOutboundStatusResponse",
    "ObservatoryServiceStub",
    "ObservatoryServiceBase",
)

from dataclasses import dataclass
from typing import TYPE_CHECKING

import betterproto2
import grpclib
from betterproto2.grpc.grpclib_server import ServiceBase

from ......message_pool import default_message_pool

if TYPE_CHECKING:
    import grpclib.server
    from betterproto2.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.core.app.observatory.command", "Config", Config
)


@dataclass(eq=False, repr=False)
class GetOutboundStatusRequest(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.core.app.observatory.command",
    "GetOutboundStatusRequest",
    GetOutboundStatusRequest,
)


@dataclass(eq=False, repr=False)
class GetOutboundStatusResponse(betterproto2.Message):
    status: "__observatory__.ObservationResult | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.core.app.observatory.command",
    "GetOutboundStatusResponse",
    GetOutboundStatusResponse,
)


class ObservatoryServiceStub(betterproto2.ServiceStub):
    async def get_outbound_status(
        self,
        message: "GetOutboundStatusRequest | None" = None,
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "GetOutboundStatusResponse":
        if message is None:
            message = GetOutboundStatusRequest()

        return await self._unary_unary(
            "/xray.core.app.observatory.command.ObservatoryService/GetOutboundStatus",
            message,
            GetOutboundStatusResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


from ... import observatory as __observatory__


class ObservatoryServiceBase(ServiceBase):
    async def get_outbound_status(
        self, message: "GetOutboundStatusRequest"
    ) -> "GetOutboundStatusResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_outbound_status(
        self,
        stream: "grpclib.server.Stream[GetOutboundStatusRequest, GetOutboundStatusResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_outbound_status(request)
        await stream.send_message(response)

    def __mapping__(self) -> "dict[str, grpclib.const.Handler]":
        return {
            "/xray.core.app.observatory.command.ObservatoryService/GetOutboundStatus": grpclib.const.Handler(
                self.__rpc_get_outbound_status,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetOutboundStatusRequest,
                GetOutboundStatusResponse,
            ),
        }



from enum import StrEnum

from xray_cfo.models.xray.enums import Network

from .._base import XBase


class SsMethod(StrEnum):
    aes_256 = "aes-256-gcm"
    aes_128 = "aes-128-gcm"
    chacha20_poly1305 = "chacha20-poly1305"
    chacha20_ietf_poly1305 = "chacha20-ietf-poly1305"
    xchacha20_poly1305 = "xchacha20-poly1305"
    xchacha20_ietf_poly1305 = "xchacha20-ietf-poly1305"

    blake3_aes_128 = "2022-blake3-aes-128-gcm"
    blake3_aes_256 = "2022-blake3-aes-256-gcm"

    blake3_chacha20_poly1305 = "blake3_chacha20-poly1305"


class SsClint(XBase):
    password: str
    method: SsMethod | None
    level: int | None
    email: str | None


class XShadowsocksIn(XBase):
    network: Network = Network.tcp
    method: SsMethod | None
    password: str
    level: int = 0
    email: str | None
    clients: list[SsClint] | None



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proxy/socks/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "AuthType",
    "Account",
    "ClientConfig",
    "ServerConfig",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class AuthType(betterproto2.Enum):
    """
    AuthType is the authentication type of Socks proxy.
    """

    NO_AUTH = 0
    """
    NO_AUTH is for anonymous authentication.
    """

    PASSWORD = 1
    """
    PASSWORD is for username/password authentication.
    """


@dataclass(eq=False, repr=False)
class Account(betterproto2.Message):
    """
    Account represents a Socks account.
    """

    username: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    password: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)


default_message_pool.register_message("xray.proxy.socks", "Account", Account)


@dataclass(eq=False, repr=False)
class ClientConfig(betterproto2.Message):
    """
    ClientConfig is the protobuf config for Socks client.
    """

    server: "list[__common__protocol__.ServerEndpoint]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )
    """
    Sever is a list of Socks server addresses.
    """


default_message_pool.register_message("xray.proxy.socks", "ClientConfig", ClientConfig)


@dataclass(eq=False, repr=False)
class ServerConfig(betterproto2.Message):
    """
    ServerConfig is the protobuf config for Socks server.
    """

    auth_type: "AuthType" = betterproto2.field(
        1, betterproto2.TYPE_ENUM, default_factory=lambda: AuthType(0)
    )

    accounts: "dict[str, str]" = betterproto2.field(
        2,
        betterproto2.TYPE_MAP,
        map_meta=betterproto2.map_meta(
            betterproto2.TYPE_STRING, betterproto2.TYPE_STRING
        ),
    )

    address: "__common__net__.IpOrDomain | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )

    udp_enabled: "bool" = betterproto2.field(4, betterproto2.TYPE_BOOL)

    user_level: "int" = betterproto2.field(6, betterproto2.TYPE_UINT32)


default_message_pool.register_message("xray.proxy.socks", "ServerConfig", ServerConfig)


from ...common import net as __common__net__
from ...common import protocol as __common__protocol__



from pydantic import Field

from .._base import XBase


class SocksAccount(XBase):
    user: str
    password: str = Field(serialization_alias="pass")


class XSocksIn(XBase):
    auth: str
    accounts: list[SocksAccount]
    udp: bool = False
    ip: str
    user_level: int = Field(0, serialization_alias="userLevel")



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proxy/vless/encoding/addons.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = ("Addons",)

from dataclasses import dataclass

import betterproto2

from .....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Addons(betterproto2.Message):
    flow: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    seed: "bytes" = betterproto2.field(2, betterproto2.TYPE_BYTES)


default_message_pool.register_message("xray.proxy.vless.encoding", "Addons", Addons)



from enum import StrEnum

from pydantic import Field

from .._base import XBase


class SockoptTProxy(StrEnum):
    redirect = "redirect"
    tproxy = "tproxy"
    off = "off"


class SockoptDomainStrategy(StrEnum):
    AsIs = "AsIs"
    UseIP = "UseIP"
    UseIPv6v4 = "UseIPv6v4"
    UseIPv6 = "UseIPv6"
    UseIPv4v6 = "UseIPv4v6"
    UseIPv4 = "UseIPv4"
    ForceIP = "ForceIP"
    ForceIPv6v4 = "ForceIPv6v4"
    ForceIPv6 = "ForceIPv6"
    ForceIPv4v6 = "ForceIPv4v6"
    ForceIPv4 = "ForceIPv4"


class CustomSockopt(XBase):
    system: str
    type: str
    level: str
    opt: str
    value: str


class Sockopt(XBase):
    mark: int
    tcp_max_seg: int = Field(1440, serialization_alias="tcpMaxSeg")
    tcp_fast_open: bool | int = Field(False, serialization_alias="tcpFastOpen")
    tproxy: SockoptTProxy = SockoptTProxy.off
    domain_strategy: SockoptDomainStrategy = Field(SockoptDomainStrategy.AsIs, serialization_alias="domainStrategy")
    dialer_proxy: str = Field("", serialization_alias="dialerProxy")
    accept_proxy_protocol: bool = Field(False, serialization_alias="acceptProxyProtocol")
    tcp_keep_alive_interval: int = Field(0, serialization_alias="tcpKeepAliveInterval")
    tcp_keep_alive_idle: int = Field(300, serialization_alias="tcpKeepAliveIdle")
    tcp_user_timeout: int = Field(10000, serialization_alias="tcpUserTimeout")
    tcpcongestion: str = "bbr"
    interface: int = ""
    v6Only: bool = Field(False, serialization_alias="V6Only")
    tcp_window_clamp: int | None = Field(None, serialization_alias="tcpWindowClamp")
    tcp_mptcp: bool = Field(False, serialization_alias="tcpMptcp")
    tcp_no_delay: bool = Field(False, serialization_alias="tcpNoDelay")
    custom_sockopt: list[CustomSockopt] | None = Field(None, serialization_alias="customSockopt")



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: common/net/address.proto, common/net/destination.proto, common/net/network.proto, common/net/port.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Network",
    "Endpoint",
    "IpOrDomain",
    "NetworkList",
    "PortList",
    "PortRange",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class Network(betterproto2.Enum):
    Unknown = 0

    TCP = 2

    UDP = 3

    UNIX = 4


@dataclass(eq=False, repr=False)
class Endpoint(betterproto2.Message):
    """
    Endpoint of a network connection.
    """

    network: "Network" = betterproto2.field(
        1, betterproto2.TYPE_ENUM, default_factory=lambda: Network(0)
    )

    address: "IpOrDomain | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )

    port: "int" = betterproto2.field(3, betterproto2.TYPE_UINT32)


default_message_pool.register_message("xray.common.net", "Endpoint", Endpoint)


@dataclass(eq=False, repr=False)
class IpOrDomain(betterproto2.Message):
    """
    Address of a network host. It may be either an IP address or a domain
    address.

    Oneofs:
        - address:
    """

    ip: "bytes | None" = betterproto2.field(
        1, betterproto2.TYPE_BYTES, optional=True, group="address"
    )
    """
    IP address. Must by either 4 or 16 bytes.
    """

    domain: "str | None" = betterproto2.field(
        2, betterproto2.TYPE_STRING, optional=True, group="address"
    )
    """
    Domain address.
    """


default_message_pool.register_message("xray.common.net", "IPOrDomain", IpOrDomain)


@dataclass(eq=False, repr=False)
class NetworkList(betterproto2.Message):
    """
    NetworkList is a list of Networks.
    """

    network: "list[Network]" = betterproto2.field(
        1, betterproto2.TYPE_ENUM, repeated=True
    )


default_message_pool.register_message("xray.common.net", "NetworkList", NetworkList)


@dataclass(eq=False, repr=False)
class PortList(betterproto2.Message):
    """
    PortList is a list of ports.
    """

    range: "list[PortRange]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message("xray.common.net", "PortList", PortList)


@dataclass(eq=False, repr=False)
class PortRange(betterproto2.Message):
    """
    PortRange represents a range of ports.
    """

    from_: "int" = betterproto2.field(1, betterproto2.TYPE_UINT32)
    """
    The port that this range starts from.
    """

    to: "int" = betterproto2.field(2, betterproto2.TYPE_UINT32)
    """
    The port that this range ends with (inclusive).
    """


default_message_pool.register_message("xray.common.net", "PortRange", PortRange)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/log/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "LogType",
    "Config",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class LogType(betterproto2.Enum):
    None_ = 0

    Console = 1

    File = 2

    Event = 3


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    error_log_type: "LogType" = betterproto2.field(
        1, betterproto2.TYPE_ENUM, default_factory=lambda: LogType(0)
    )

    error_log_level: "__common__log__.Severity" = betterproto2.field(
        2, betterproto2.TYPE_ENUM, default_factory=lambda: __common__log__.Severity(0)
    )

    error_log_path: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)

    access_log_type: "LogType" = betterproto2.field(
        4, betterproto2.TYPE_ENUM, default_factory=lambda: LogType(0)
    )

    access_log_path: "str" = betterproto2.field(5, betterproto2.TYPE_STRING)

    enable_dns_log: "bool" = betterproto2.field(6, betterproto2.TYPE_BOOL)

    mask_address: "str" = betterproto2.field(7, betterproto2.TYPE_STRING)


default_message_pool.register_message("xray.app.log", "Config", Config)


from ...common import log as __common__log__



from grpclib.exceptions import GRPCError
from xray_cfo.xray.app.proxyman.command import AddInboundRequest

from .._base import BaseService


class HandlerAddInboundService(BaseService):
    async def _add_inbound(self, message: AddInboundRequest):
        try:
            return await self.handler_stub.add_inbound(message)
        except GRPCError:
            return None



from types import TracebackType

from grpclib.client import Channel

from xray_cfo.api_services import APIService


class XrayClient(APIService):

    def __init__(self, host: str, port: int):
        self.host = host
        self.port = port
        self._channel = Channel(
            host=host,
            port=port,
        )
        super().__init__()

    async def __aenter__(self):
        return self

    async def __aexit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None:
        self.close()

    def close(self):
        self._channel.close()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/stats/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "ChannelConfig",
    "Config",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class ChannelConfig(betterproto2.Message):
    blocking: "bool" = betterproto2.field(1, betterproto2.TYPE_BOOL)

    subscriber_limit: "int" = betterproto2.field(2, betterproto2.TYPE_INT32)

    buffer_size: "int" = betterproto2.field(3, betterproto2.TYPE_INT32)


default_message_pool.register_message("xray.app.stats", "ChannelConfig", ChannelConfig)


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    pass


default_message_pool.register_message("xray.app.stats", "Config", Config)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proxy/dokodemo/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = ("Config",)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    address: "__common__net__.IpOrDomain | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )

    port: "int" = betterproto2.field(2, betterproto2.TYPE_UINT32)

    networks: "list[__common__net__.Network]" = betterproto2.field(
        7, betterproto2.TYPE_ENUM, repeated=True
    )
    """
    List of networks that the Dokodemo accepts.
    """

    follow_redirect: "bool" = betterproto2.field(5, betterproto2.TYPE_BOOL)

    user_level: "int" = betterproto2.field(6, betterproto2.TYPE_UINT32)


default_message_pool.register_message("xray.proxy.dokodemo", "Config", Config)


from ...common import net as __common__net__



from enum import StrEnum

from pydantic import Field

from .._base import XBase
from .grpc import XGRPCSettings
from .httpupgrade import XHttpUpgradeSettings
from .kcp import XKCPSettings
from .raw import XRawSettings
from .reality import XRealitySettingsIN, XRealitySettingsOUT
from .socopt import Sockopt
from .tls import XTLSSettings
from .ws import XWebSocketSettings
from .xhttp import XXHttpSettings


class StreamNetwork(StrEnum):
    raw = "raw"
    xhttp = "xhttp"
    kcp = "kcp"
    grpc = "grpc"
    ws = "ws"
    httpupgrade = "httpupgrade"


class Security(StrEnum):
    none = "none"
    tls = "tls"
    reality = "reality"


class XStreamSettings(XBase):
    network: StreamNetwork = StreamNetwork.raw
    security: Security = Security.none
    tls_settings: XTLSSettings | None = Field(None, serialization_alias="tlsSettings")
    reality_settings: XRealitySettingsIN | XRealitySettingsOUT | None = Field(
        None, serialization_alias="realitySettings"
    )
    raw_settings: XRawSettings | None = Field(None, serialization_alias="rawSettings")
    xhttp_settings: XXHttpSettings | None = Field(
        None, serialization_alias="xhttpSettings"
    )
    kcp_settings: XKCPSettings | None = Field(None, serialization_alias="kcpSettings")
    grpc_settings: XGRPCSettings | None = Field(
        None, serialization_alias="grpcSettings"
    )
    ws_settings: XWebSocketSettings | None = Field(
        None, serialization_alias="wsSettings"
    )
    httpupgrade_settings: XHttpUpgradeSettings | None = Field(
        None, serialization_alias="httpupgradeSettings"
    )
    sockopt: Sockopt | None = Field(None)



from pydantic import Field

from xray_cfo.models.xray.enums import Network

from .._base import XBase


class XDokoDemoIn(XBase):
    address: str
    port: int | str
    network: Network
    follow_redirect: bool = Field(serialization_alias="followRedirect")
    user_level: int = Field(serialization_alias="userLevel")



import base64
import math
import secrets
import socket
import uuid

import betterproto2
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import x25519

from xray_cfo.message_pool import default_message_pool
from xray_cfo.xray.common.serial import TypedMessage


def to_typed_message(message: betterproto2.Message):
    return TypedMessage(
        type=message.__class__.__name__, value=message.SerializeToString()
    )


def to_typed_message_v2(message: betterproto2.Message):
    message_type = message.__class__
    type_url = default_message_pool.type_to_url.get(message_type)
    if "/" in type_url:
        type_url = type_url.split("/")[-1]
    return TypedMessage(
        type=type_url,
        value=message.SerializeToString(),
    )


def get_message_type(message: betterproto2.Message):
    message_type = message.__class__
    type_url = default_message_pool.type_to_url.get(message_type)
    if "/" in type_url:
        type_url = type_url.split("/")[-1]
    return type_url


def ip2bytes(ip: str):
    return bytes([int(i) for i in ip.split(".")])


def generate_random_tag():
    return secrets.token_urlsafe()


def generate_random_name(hex_count=8):
    return f"{secrets.token_hex(hex_count)}"


def generate_random_user_id():
    return uuid.uuid4().hex


def generate_random_email(tld="vump.com", hex_count=8):
    return f"{generate_random_name(hex_count)}@{tld}"


def generate_short_id() -> bytes:
    return secrets.token_bytes(8)


def generate_short_ids(count_ids: int = 10) -> list[bytes]:
    return [generate_short_id() for _ in range(count_ids)]


def generate_random_port():
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as tcp:
        tcp.bind(("", 0))
        addr, port = tcp.getsockname()
    return port


def human_readable_bytes(size_bytes):
    if not size_bytes:
        return "0 B"
    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)
    return f"{s} {size_name[i]}"


def generate_x25519_keys() -> tuple[bytes, bytes]:
    private_key = x25519.X25519PrivateKey.generate()

    public_key = private_key.public_key()

    private_bytes = private_key.private_bytes(
        encoding=serialization.Encoding.Raw,
        format=serialization.PrivateFormat.Raw,
        encryption_algorithm=serialization.NoEncryption(),
    )
    public_bytes = public_key.public_bytes(
        encoding=serialization.Encoding.Raw, format=serialization.PublicFormat.Raw
    )

    return private_bytes, public_bytes


def generate_x25519_keys():
    priv = x25519.X25519PrivateKey.generate()
    pub = priv.public_key()
    priv_bytes = priv.private_bytes(
        encoding=serialization.Encoding.Raw,
        format=serialization.PrivateFormat.Raw,
        encryption_algorithm=serialization.NoEncryption(),
    )
    pub_bytes = pub.public_bytes(
        encoding=serialization.Encoding.Raw, format=serialization.PublicFormat.Raw
    )
    import base64

    # pub_str = base64.urlsafe_b64encode(pub_bytes).decode()[:-1]
    # priv_str = base64.urlsafe_b64encode(priv_bytes).decode()[:-1]

    return priv_bytes, pub_bytes


def generate_wireguard_keys(use_hex: bool = False) -> tuple[str, str]:
    private_key = x25519.X25519PrivateKey.generate()
    public_key = private_key.public_key()
    private_bytes = private_key.private_bytes(
        encoding=serialization.Encoding.Raw,
        format=serialization.PrivateFormat.Raw,
        encryption_algorithm=serialization.NoEncryption(),
    )
    public_bytes = public_key.public_bytes(
        encoding=serialization.Encoding.Raw, format=serialization.PublicFormat.Raw
    )
    if use_hex:
        private_key_str = private_bytes.hex()
        public_key_str = public_bytes.hex()
    else:
        private_key_str = base64.b64encode(private_bytes).decode("utf-8")
        public_key_str = base64.b64encode(public_bytes).decode("utf-8")

    return private_key_str, public_key_str


def generate_vless_reality_link(
    user_id: str,
    domain_or_ip: str,
    public_key: bytes,
    short_id: bytes,
    port: int = 443,
    email: str = "user",
    sni: str = "amazon.com",
    spider_x: str = "/",
    flow: str = "xtls-rprx-vision",
    fingerprint: str = "chrome",
    header_type: str = "none",
) -> str:
    """
    Генерирует VLESS ссылку для Reality + Vision + TCP.

    :param user_id: UUID пользователя (str)
    :param domain_or_ip: Хост или IP сервера
    :param public_key: Публичный ключ Reality
    :param short_id: Short ID (обычно один из сгенерированных)
    :param email: Комментарий (опционально)
    :param sni: SNI/DEST, по умолчанию amazon.com
    :param spider_x: SPX путь, по умолчанию '/'
    :param flow: Потоковая схема, по умолчанию xtls-rprx-vision
    :param fingerprint: Браузерный отпечаток (chrome, firefox и т.д.)
    :param port: Порт, обычно 443
    :param header_type: Тип заголовка TCP (обычно none)
    :return: Готовая VLESS-ссылка
    """
    # public_key = base64.urlsafe_b64encode(public_key).rstrip(b"=").decode()
    # public_key = base64.urlsafe_b64encode(public_key).decode()[:-1]
    # short_id = base64.urlsafe_b64encode(short_id).rstrip(b"=").decode()
    return (
        f"vless://{user_id}@{domain_or_ip}:{port}"
        f"?encryption=none"
        f"&flow={flow}"
        f"&security=reality"
        f"&sni={sni}"
        f"&fp={fingerprint}"
        f"&pbk={public_key}"
        f"&sid={short_id}"
        f"&type=tcp"
        f"&headerType={header_type}"
        f"&spx={spider_x}"
        f"#{email}"
    )


def hex_to_base64(s: str) -> str:
    return base64.b64encode(bytes.fromhex(s)).decode()


def generate_wireguard_client_config(
    client_private_key: str,
    client_ip: str,
    server_public_key: str,
    server_endpoint: str,
    mtu: int = 1420,
    allowed_ips: str = "0.0.0.0/0",
    dns: str = "1.1.1.1",
    keep_alive: int = 25,
) -> str:
    """
    Сгенерировать конфигурацию клиента WireGuard.

    :param client_private_key: Приватный ключ клиента (Base64)
    :param client_ip: IP клиента в сети VPN (например, "10.0.0.2")
    :param server_public_key: Публичный ключ сервера (Base64)
    :param server_endpoint: Адрес сервера и порт (например, "vpn.example.com:443")
    :param mtu: MTU интерфейса
    :param allowed_ips: Разрешённые маршруты (по умолчанию весь трафик)
    :param dns: DNS-сервер
    :param keep_alive: Интервал keep-alive сообщений (в секундах)
    :return: Строка WireGuard-конфига
    """
    return f"""[Interface]
PrivateKey = {hex_to_base64(client_private_key)}
Address = {client_ip}/32
DNS = {dns}
MTU = {mtu}

[Peer]
PublicKey = {hex_to_base64(server_public_key)}
Endpoint = {server_endpoint}
AllowedIPs = {allowed_ips}
"""



from grpclib.exceptions import GRPCError

from .._base import BaseService


class HandlerListBoundService(BaseService):

    async def list_inbounds(self):
        try:
            return (await self.handler_stub.list_inbounds()).inbounds
        except GRPCError:
            return None

    async def list_outbounds(self):
        try:
            return (await self.handler_stub.list_outbounds()).outbounds
        except GRPCError:
            return None



import ipaddress
import re
from enum import StrEnum

from pydantic import BaseModel, Field, field_validator

from ._base import XBase


class ApiService(StrEnum):
    handler = "HandlerService"
    logger = "LoggerService"
    stats = "StatsService"
    routing = "RoutingService"
    reflection = "ReflectionService"


LISTEN_REGEX = re.compile(r"^(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}|\[.*\]|\w+):\d+$")


class XApi(XBase):
    tag: str
    listen: str | None
    service: list[ApiService] = Field(default_factory=list)

    @field_validator("listen")
    def validate_listen_format(cls, v):
        if v is None:
            return v

        if not LISTEN_REGEX.match(v):
            raise ValueError("listen must be in format 'ip:port'")

        try:
            ip, port = v.rsplit(":", 1)
            ip_cleaned = ip[1:-1] if ip.startswith("[") and ip.endswith("]") else ip
            ipaddress.ip_address(ip_cleaned)
        except Exception as e:
            raise ValueError(f"Invalid IP address or port: {e}")

        return v



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/router/command/command.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "AddRuleRequest",
    "AddRuleResponse",
    "BalancerMsg",
    "Config",
    "GetBalancerInfoRequest",
    "GetBalancerInfoResponse",
    "OverrideBalancerTargetRequest",
    "OverrideBalancerTargetResponse",
    "OverrideInfo",
    "PrincipleTargetInfo",
    "RemoveRuleRequest",
    "RemoveRuleResponse",
    "RoutingContext",
    "SubscribeRoutingStatsRequest",
    "TestRouteRequest",
    "RoutingServiceStub",
    "RoutingServiceBase",
)

from collections.abc import AsyncIterator
from dataclasses import dataclass
from typing import TYPE_CHECKING

import betterproto2
import grpclib
from betterproto2.grpc.grpclib_server import ServiceBase

from .....message_pool import default_message_pool

if TYPE_CHECKING:
    import grpclib.server
    from betterproto2.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class AddRuleRequest(betterproto2.Message):
    config: "___common__serial__.TypedMessage | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )

    should_append: "bool" = betterproto2.field(2, betterproto2.TYPE_BOOL)


default_message_pool.register_message(
    "xray.app.router.command", "AddRuleRequest", AddRuleRequest
)


@dataclass(eq=False, repr=False)
class AddRuleResponse(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.router.command", "AddRuleResponse", AddRuleResponse
)


@dataclass(eq=False, repr=False)
class BalancerMsg(betterproto2.Message):
    override: "OverrideInfo | None" = betterproto2.field(
        5, betterproto2.TYPE_MESSAGE, optional=True
    )

    principle_target: "PrincipleTargetInfo | None" = betterproto2.field(
        6, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.app.router.command", "BalancerMsg", BalancerMsg
)


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    pass


default_message_pool.register_message("xray.app.router.command", "Config", Config)


@dataclass(eq=False, repr=False)
class GetBalancerInfoRequest(betterproto2.Message):
    tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.app.router.command", "GetBalancerInfoRequest", GetBalancerInfoRequest
)


@dataclass(eq=False, repr=False)
class GetBalancerInfoResponse(betterproto2.Message):
    balancer: "BalancerMsg | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.app.router.command", "GetBalancerInfoResponse", GetBalancerInfoResponse
)


@dataclass(eq=False, repr=False)
class OverrideBalancerTargetRequest(betterproto2.Message):
    balancer_tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    target: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.app.router.command",
    "OverrideBalancerTargetRequest",
    OverrideBalancerTargetRequest,
)


@dataclass(eq=False, repr=False)
class OverrideBalancerTargetResponse(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.router.command",
    "OverrideBalancerTargetResponse",
    OverrideBalancerTargetResponse,
)


@dataclass(eq=False, repr=False)
class OverrideInfo(betterproto2.Message):
    target: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.app.router.command", "OverrideInfo", OverrideInfo
)


@dataclass(eq=False, repr=False)
class PrincipleTargetInfo(betterproto2.Message):
    tag: "list[str]" = betterproto2.field(1, betterproto2.TYPE_STRING, repeated=True)


default_message_pool.register_message(
    "xray.app.router.command", "PrincipleTargetInfo", PrincipleTargetInfo
)


@dataclass(eq=False, repr=False)
class RemoveRuleRequest(betterproto2.Message):
    rule_tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.app.router.command", "RemoveRuleRequest", RemoveRuleRequest
)


@dataclass(eq=False, repr=False)
class RemoveRuleResponse(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.router.command", "RemoveRuleResponse", RemoveRuleResponse
)


@dataclass(eq=False, repr=False)
class RoutingContext(betterproto2.Message):
    """
    RoutingContext is the context with information relative to routing process.
    It conforms to the structure of xray.features.routing.Context and
    xray.features.routing.Route.
    """

    inbound_tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    network: "___common__net__.Network" = betterproto2.field(
        2, betterproto2.TYPE_ENUM, default_factory=lambda: ___common__net__.Network(0)
    )

    source_i_ps: "list[bytes]" = betterproto2.field(
        3, betterproto2.TYPE_BYTES, repeated=True
    )

    target_i_ps: "list[bytes]" = betterproto2.field(
        4, betterproto2.TYPE_BYTES, repeated=True
    )

    source_port: "int" = betterproto2.field(5, betterproto2.TYPE_UINT32)

    target_port: "int" = betterproto2.field(6, betterproto2.TYPE_UINT32)

    target_domain: "str" = betterproto2.field(7, betterproto2.TYPE_STRING)

    protocol: "str" = betterproto2.field(8, betterproto2.TYPE_STRING)

    user: "str" = betterproto2.field(9, betterproto2.TYPE_STRING)

    attributes: "dict[str, str]" = betterproto2.field(
        10,
        betterproto2.TYPE_MAP,
        map_meta=betterproto2.map_meta(
            betterproto2.TYPE_STRING, betterproto2.TYPE_STRING
        ),
    )

    outbound_group_tags: "list[str]" = betterproto2.field(
        11, betterproto2.TYPE_STRING, repeated=True
    )

    outbound_tag: "str" = betterproto2.field(12, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.app.router.command", "RoutingContext", RoutingContext
)


@dataclass(eq=False, repr=False)
class SubscribeRoutingStatsRequest(betterproto2.Message):
    """
    SubscribeRoutingStatsRequest subscribes to routing statistics channel if
    opened by xray-core.
    * FieldSelectors selects a subset of fields in routing statistics to return.
    Valid selectors:
     - inbound: Selects connection's inbound tag.
     - network: Selects connection's network.
     - ip: Equivalent as "ip_source" and "ip_target", selects both source and
     target IP.
     - port: Equivalent as "port_source" and "port_target", selects both source
     and target port.
     - domain: Selects target domain.
     - protocol: Select connection's protocol.
     - user: Select connection's inbound user email.
     - attributes: Select connection's additional attributes.
     - outbound: Equivalent as "outbound" and "outbound_group", select both
     outbound tag and outbound group tags.
    * If FieldSelectors is left empty, all fields will be returned.
    """

    field_selectors: "list[str]" = betterproto2.field(
        1, betterproto2.TYPE_STRING, repeated=True
    )


default_message_pool.register_message(
    "xray.app.router.command",
    "SubscribeRoutingStatsRequest",
    SubscribeRoutingStatsRequest,
)


@dataclass(eq=False, repr=False)
class TestRouteRequest(betterproto2.Message):
    """
    TestRouteRequest manually tests a routing result according to the routing
    context message.
    * RoutingContext is the routing message without outbound information.
    * FieldSelectors selects the fields to return in the routing result. All
    fields are returned if left empty.
    * PublishResult broadcasts the routing result to routing statistics channel
    if set true.
    """

    routing_context: "RoutingContext | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )

    field_selectors: "list[str]" = betterproto2.field(
        2, betterproto2.TYPE_STRING, repeated=True
    )

    publish_result: "bool" = betterproto2.field(3, betterproto2.TYPE_BOOL)


default_message_pool.register_message(
    "xray.app.router.command", "TestRouteRequest", TestRouteRequest
)


class RoutingServiceStub(betterproto2.ServiceStub):
    async def subscribe_routing_stats(
        self,
        message: "SubscribeRoutingStatsRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "AsyncIterator[RoutingContext]":
        async for response in self._unary_stream(
            "/xray.app.router.command.RoutingService/SubscribeRoutingStats",
            message,
            RoutingContext,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    async def test_route(
        self,
        message: "TestRouteRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "RoutingContext":
        return await self._unary_unary(
            "/xray.app.router.command.RoutingService/TestRoute",
            message,
            RoutingContext,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_balancer_info(
        self,
        message: "GetBalancerInfoRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "GetBalancerInfoResponse":
        return await self._unary_unary(
            "/xray.app.router.command.RoutingService/GetBalancerInfo",
            message,
            GetBalancerInfoResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def override_balancer_target(
        self,
        message: "OverrideBalancerTargetRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "OverrideBalancerTargetResponse":
        return await self._unary_unary(
            "/xray.app.router.command.RoutingService/OverrideBalancerTarget",
            message,
            OverrideBalancerTargetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def add_rule(
        self,
        message: "AddRuleRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "AddRuleResponse":
        return await self._unary_unary(
            "/xray.app.router.command.RoutingService/AddRule",
            message,
            AddRuleResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def remove_rule(
        self,
        message: "RemoveRuleRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "RemoveRuleResponse":
        return await self._unary_unary(
            "/xray.app.router.command.RoutingService/RemoveRule",
            message,
            RemoveRuleResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


from ....common import net as ___common__net__
from ....common import serial as ___common__serial__


class RoutingServiceBase(ServiceBase):
    async def subscribe_routing_stats(
        self, message: "SubscribeRoutingStatsRequest"
    ) -> "AsyncIterator[RoutingContext]":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def test_route(self, message: "TestRouteRequest") -> "RoutingContext":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_balancer_info(
        self, message: "GetBalancerInfoRequest"
    ) -> "GetBalancerInfoResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def override_balancer_target(
        self, message: "OverrideBalancerTargetRequest"
    ) -> "OverrideBalancerTargetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def add_rule(self, message: "AddRuleRequest") -> "AddRuleResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def remove_rule(self, message: "RemoveRuleRequest") -> "RemoveRuleResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_subscribe_routing_stats(
        self,
        stream: "grpclib.server.Stream[SubscribeRoutingStatsRequest, RoutingContext]",
    ) -> None:
        request = await stream.recv_message()
        await self._call_rpc_handler_server_stream(
            self.subscribe_routing_stats,
            stream,
            request,
        )

    async def __rpc_test_route(
        self, stream: "grpclib.server.Stream[TestRouteRequest, RoutingContext]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.test_route(request)
        await stream.send_message(response)

    async def __rpc_get_balancer_info(
        self,
        stream: "grpclib.server.Stream[GetBalancerInfoRequest, GetBalancerInfoResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_balancer_info(request)
        await stream.send_message(response)

    async def __rpc_override_balancer_target(
        self,
        stream: "grpclib.server.Stream[OverrideBalancerTargetRequest, OverrideBalancerTargetResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.override_balancer_target(request)
        await stream.send_message(response)

    async def __rpc_add_rule(
        self, stream: "grpclib.server.Stream[AddRuleRequest, AddRuleResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.add_rule(request)
        await stream.send_message(response)

    async def __rpc_remove_rule(
        self, stream: "grpclib.server.Stream[RemoveRuleRequest, RemoveRuleResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.remove_rule(request)
        await stream.send_message(response)

    def __mapping__(self) -> "dict[str, grpclib.const.Handler]":
        return {
            "/xray.app.router.command.RoutingService/SubscribeRoutingStats": grpclib.const.Handler(
                self.__rpc_subscribe_routing_stats,
                grpclib.const.Cardinality.UNARY_STREAM,
                SubscribeRoutingStatsRequest,
                RoutingContext,
            ),
            "/xray.app.router.command.RoutingService/TestRoute": grpclib.const.Handler(
                self.__rpc_test_route,
                grpclib.const.Cardinality.UNARY_UNARY,
                TestRouteRequest,
                RoutingContext,
            ),
            "/xray.app.router.command.RoutingService/GetBalancerInfo": grpclib.const.Handler(
                self.__rpc_get_balancer_info,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetBalancerInfoRequest,
                GetBalancerInfoResponse,
            ),
            "/xray.app.router.command.RoutingService/OverrideBalancerTarget": grpclib.const.Handler(
                self.__rpc_override_balancer_target,
                grpclib.const.Cardinality.UNARY_UNARY,
                OverrideBalancerTargetRequest,
                OverrideBalancerTargetResponse,
            ),
            "/xray.app.router.command.RoutingService/AddRule": grpclib.const.Handler(
                self.__rpc_add_rule,
                grpclib.const.Cardinality.UNARY_UNARY,
                AddRuleRequest,
                AddRuleResponse,
            ),
            "/xray.app.router.command.RoutingService/RemoveRule": grpclib.const.Handler(
                self.__rpc_remove_rule,
                grpclib.const.Cardinality.UNARY_UNARY,
                RemoveRuleRequest,
                RemoveRuleResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/dns/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "DomainMatchingType",
    "QueryStrategy",
    "Config",
    "ConfigHostMapping",
    "NameServer",
    "NameServerOriginalRule",
    "NameServerPriorityDomain",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class DomainMatchingType(betterproto2.Enum):
    Full = 0

    Subdomain = 1

    Keyword = 2

    Regex = 3


class QueryStrategy(betterproto2.Enum):
    USE_IP = 0

    USE_IP4 = 1

    USE_IP6 = 2


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    name_server: "list[NameServer]" = betterproto2.field(
        5, betterproto2.TYPE_MESSAGE, repeated=True
    )
    """
    NameServer list used by this DNS client.
    A special value 'localhost' as a domain address can be set to use DNS on local system.
    """

    client_ip: "bytes" = betterproto2.field(3, betterproto2.TYPE_BYTES)
    """
    Client IP for EDNS client subnet. Must be 4 bytes (IPv4) or 16 bytes
    (IPv6).
    """

    static_hosts: "list[ConfigHostMapping]" = betterproto2.field(
        4, betterproto2.TYPE_MESSAGE, repeated=True
    )

    tag: "str" = betterproto2.field(6, betterproto2.TYPE_STRING)
    """
    Tag is the inbound tag of DNS client.
    """

    disable_cache: "bool" = betterproto2.field(8, betterproto2.TYPE_BOOL)
    """
    DisableCache disables DNS cache
    """

    query_strategy: "QueryStrategy" = betterproto2.field(
        9, betterproto2.TYPE_ENUM, default_factory=lambda: QueryStrategy(0)
    )

    disable_fallback: "bool" = betterproto2.field(10, betterproto2.TYPE_BOOL)

    disable_fallback_if_match: "bool" = betterproto2.field(11, betterproto2.TYPE_BOOL)


default_message_pool.register_message("xray.app.dns", "Config", Config)


@dataclass(eq=False, repr=False)
class ConfigHostMapping(betterproto2.Message):
    type: "DomainMatchingType" = betterproto2.field(
        1, betterproto2.TYPE_ENUM, default_factory=lambda: DomainMatchingType(0)
    )

    domain: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)

    ip: "list[bytes]" = betterproto2.field(3, betterproto2.TYPE_BYTES, repeated=True)

    proxied_domain: "str" = betterproto2.field(4, betterproto2.TYPE_STRING)
    """
    ProxiedDomain indicates the mapped domain has the same IP address on this
    domain. Xray will use this domain for IP queries.
    """


default_message_pool.register_message(
    "xray.app.dns", "Config.HostMapping", ConfigHostMapping
)


@dataclass(eq=False, repr=False)
class NameServer(betterproto2.Message):
    address: "__common__net__.Endpoint | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )

    client_ip: "bytes" = betterproto2.field(5, betterproto2.TYPE_BYTES)

    skip_fallback: "bool" = betterproto2.field(6, betterproto2.TYPE_BOOL)

    prioritized_domain: "list[NameServerPriorityDomain]" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, repeated=True
    )

    geoip: "list[_router__.GeoIp]" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, repeated=True
    )

    original_rules: "list[NameServerOriginalRule]" = betterproto2.field(
        4, betterproto2.TYPE_MESSAGE, repeated=True
    )

    query_strategy: "QueryStrategy" = betterproto2.field(
        7, betterproto2.TYPE_ENUM, default_factory=lambda: QueryStrategy(0)
    )

    allow_unexpected_i_ps: "bool" = betterproto2.field(8, betterproto2.TYPE_BOOL)

    tag: "str" = betterproto2.field(9, betterproto2.TYPE_STRING)

    timeout_ms: "int" = betterproto2.field(10, betterproto2.TYPE_UINT64)


default_message_pool.register_message("xray.app.dns", "NameServer", NameServer)


@dataclass(eq=False, repr=False)
class NameServerOriginalRule(betterproto2.Message):
    rule: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    size: "int" = betterproto2.field(2, betterproto2.TYPE_UINT32)


default_message_pool.register_message(
    "xray.app.dns", "NameServer.OriginalRule", NameServerOriginalRule
)


@dataclass(eq=False, repr=False)
class NameServerPriorityDomain(betterproto2.Message):
    type: "DomainMatchingType" = betterproto2.field(
        1, betterproto2.TYPE_ENUM, default_factory=lambda: DomainMatchingType(0)
    )

    domain: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.app.dns", "NameServer.PriorityDomain", NameServerPriorityDomain
)


from ...common import net as __common__net__
from .. import router as _router__



import ipaddress

from pydantic import Field, field_validator, model_validator

from ._base import XBase


class XFakeDns(XBase):
    ip_pool: str = Field(serialization_alias="ipPool")
    pool_size: int = Field(65535, serialization_alias="poolSize")

    @field_validator("ip_pool")
    def validate_ip_pool(cls, v: str) -> str:
        try:
            ipaddress.ip_network(v, strict=False)
        except ValueError:
            raise ValueError(f"Invalid CIDR for ipPool: {v}")
        return v

    @model_validator(mode="after")
    def validate_pool_size(self) -> "XFakeDns":
        if self.pool_size <= 0:
            raise ValueError("poolSize must be a positive integer"
                             )
        if self.ip_pool:
            network = ipaddress.ip_network(self.ip_pool, strict=False)
            total_ips = network.num_addresses
            if self.pool_size > total_ips:
                raise ValueError(f"poolSize ({self.pool_size}) cannot exceed number of addresses in ipPool ({total_ips})")
        return self



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/dns/fakedns/fakedns.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "FakeDnsPool",
    "FakeDnsPoolMulti",
)

from dataclasses import dataclass

import betterproto2

from .....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class FakeDnsPool(betterproto2.Message):
    ip_pool: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)
    """
    CIDR of IP pool used as fake DNS IP
    """

    lru_size: "int" = betterproto2.field(2, betterproto2.TYPE_INT64)
    """
    Size of Pool for remembering relationship between domain name and IP address
    """


default_message_pool.register_message(
    "xray.app.dns.fakedns", "FakeDnsPool", FakeDnsPool
)


@dataclass(eq=False, repr=False)
class FakeDnsPoolMulti(betterproto2.Message):
    pools: "list[FakeDnsPool]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message(
    "xray.app.dns.fakedns", "FakeDnsPoolMulti", FakeDnsPoolMulti
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: transport/internet/splithttp/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "RangeConfig",
    "XmuxConfig",
)

from dataclasses import dataclass

import betterproto2

from .....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    host: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    path: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)

    mode: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)

    headers: "dict[str, str]" = betterproto2.field(
        4,
        betterproto2.TYPE_MAP,
        map_meta=betterproto2.map_meta(
            betterproto2.TYPE_STRING, betterproto2.TYPE_STRING
        ),
    )

    x_padding_bytes: "RangeConfig | None" = betterproto2.field(
        5, betterproto2.TYPE_MESSAGE, optional=True
    )

    no_grpc_header: "bool" = betterproto2.field(6, betterproto2.TYPE_BOOL)

    no_sse_header: "bool" = betterproto2.field(7, betterproto2.TYPE_BOOL)

    sc_max_each_post_bytes: "RangeConfig | None" = betterproto2.field(
        8, betterproto2.TYPE_MESSAGE, optional=True
    )

    sc_min_posts_interval_ms: "RangeConfig | None" = betterproto2.field(
        9, betterproto2.TYPE_MESSAGE, optional=True
    )

    sc_max_buffered_posts: "int" = betterproto2.field(10, betterproto2.TYPE_INT64)

    sc_stream_up_server_secs: "RangeConfig | None" = betterproto2.field(
        11, betterproto2.TYPE_MESSAGE, optional=True
    )

    xmux: "XmuxConfig | None" = betterproto2.field(
        12, betterproto2.TYPE_MESSAGE, optional=True
    )

    download_settings: "__internet__.StreamConfig | None" = betterproto2.field(
        13, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.transport.internet.splithttp", "Config", Config
)


@dataclass(eq=False, repr=False)
class RangeConfig(betterproto2.Message):
    from_: "int" = betterproto2.field(1, betterproto2.TYPE_INT32)

    to: "int" = betterproto2.field(2, betterproto2.TYPE_INT32)


default_message_pool.register_message(
    "xray.transport.internet.splithttp", "RangeConfig", RangeConfig
)


@dataclass(eq=False, repr=False)
class XmuxConfig(betterproto2.Message):
    max_concurrency: "RangeConfig | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )

    max_connections: "RangeConfig | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )

    c_max_reuse_times: "RangeConfig | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )

    h_max_request_times: "RangeConfig | None" = betterproto2.field(
        4, betterproto2.TYPE_MESSAGE, optional=True
    )

    h_max_reusable_secs: "RangeConfig | None" = betterproto2.field(
        5, betterproto2.TYPE_MESSAGE, optional=True
    )

    h_keep_alive_period: "int" = betterproto2.field(6, betterproto2.TYPE_INT64)


default_message_pool.register_message(
    "xray.transport.internet.splithttp", "XmuxConfig", XmuxConfig
)


from ... import internet as __internet__



from pydantic import Field

from .._base import XBase


def get_default_response():
    return {"type": "none"}


class BlackHoleOut(XBase):
    response: dict[str, str] = Field(default_factory=get_default_response)



from grpclib.exceptions import GRPCError

from xray_cfo.common import utils
from xray_cfo.xray.app.proxyman.command import AddUserOperation, AlterInboundRequest
from xray_cfo.xray.common.protocol import User
from xray_cfo.xray.proxy import shadowsocks, vless
from xray_cfo.xray.proxy.shadowsocks import CipherType

from .._base import BaseService


class HandlerAddClientService(BaseService):

    async def add_client_vless(
        self,
        inbound_tag: str,
        user_id: str,
        email: str,
        level: int = 0,
        flow: str = "none",
        encryption: str = "none",
    ):
        try:
            await self.handler_stub.alter_inbound(
                AlterInboundRequest(
                    tag=inbound_tag,
                    operation=utils.to_typed_message(
                        AddUserOperation(
                            user=User(
                                email=email,
                                level=level,
                                account=utils.to_typed_message(
                                    vless.Account(
                                        id=user_id,
                                        flow=flow,
                                        encryption=encryption,
                                    )
                                ),
                            )
                        )
                    ),
                )
            )
            return user_id
        except GRPCError:
            return None

    async def add_client_ss(
        self,
        inbound_tag: str,
        user_id: str,
        email: str,
        level: int = 0,
    ):
        try:
            await self.handler_stub.alter_inbound(
                AlterInboundRequest(
                    tag=inbound_tag,
                    operation=utils.to_typed_message(
                        AddUserOperation(
                            user=User(
                                email=email,
                                level=level,
                                account=utils.to_typed_message(
                                    shadowsocks.Account(
                                        password=user_id,
                                        cipher_type=CipherType.CHACHA20_POLY1305,
                                        iv_check=True,
                                    )
                                ),
                            )
                        )
                    ),
                )
            )
            return user_id
        except GRPCError:
            return None

    async def add_client_wg(self, inbound_tag: str):

        try:
            inbounds_data = await self.handler_stub.list_inbounds()
            for inbound in inbounds_data.inbounds:
                if inbound_tag == inbound.tag:
                    selected_inbound = inbound
                    break
            else:
                return

            # TODO(CFO): need edit_inbound and add peer

        except:
            return
